@@ -10,14 +10,14 @@
 %                            P       CCCC  DDDD                               %
 %                                                                             %
 %                                                                             %
-%                     Read/Write Photo CD Image Format.                       %
+%                     Read/Write Photo CD Image Format                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,12 +40,16 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/property.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/client.h"
 #include "magick/colorspace.h"
+#include "magick/colorspace-private.h"
 #include "magick/constitute.h"
 #include "magick/decorate.h"
+#include "magick/distort.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/gem.h"
@@ -55,13 +59,14 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
-#include "magick/mogrify.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/montage.h"
 #include "magick/resize.h"
-#include "magick/shear.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
 
@@ -92,25 +97,22 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image: The address of a structure of type Image.
+%    o image: the address of a structure of type Image.
 %
-%    o luma: The address of a character buffer that contains the
+%    o luma: the address of a character buffer that contains the
 %      luminance information.
 %
-%    o chroma1: The address of a character buffer that contains the
+%    o chroma1: the address of a character buffer that contains the
 %      chrominance information.
 %
-%    o chroma2: The address of a character buffer that contains the
+%    o chroma2: the address of a character buffer that contains the
 %      chrominance information.
 %
-%
-%
 */
 static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   unsigned char *chroma1,unsigned char *chroma2)
 {
-#define IsSync  ((sum & 0xffffff00) == 0xfffffe00)
-#define DecodeImageTag  "Decode/Image"
+#define IsSync(sum)  ((sum & 0xffffff00UL) == 0xfffffe00UL)
 #define PCDGetBits(n) \
 {  \
   sum=(sum << n) & 0xffffffff; \
@@ -126,8 +128,6 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     bits+=8; \
     p++; \
   } \
-  if (EOFBlob(image) != MagickFalse) \
-    break; \
 }
 
   typedef struct PCDTable
@@ -143,17 +143,10 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
       key;
   } PCDTable;
 
-  long
-    count,
-    quantum;
-
-  MagickBooleanType
-    status;
-
   PCDTable
     *pcd_table[3];
 
-  register long
+  register ssize_t
     i,
     j;
 
@@ -165,18 +158,20 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     *q;
 
   size_t
-    length;
-
-  unsigned char
-    *buffer;
-
-  unsigned long
     bits,
+    length,
     plane,
     pcd_length[3],
     row,
     sum;
 
+  ssize_t
+    count,
+    quantum;
+
+  unsigned char
+    *buffer;
+
   /*
     Initialize Huffman tables.
   */
@@ -187,7 +182,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   assert(luma != (unsigned char *) NULL);
   assert(chroma1 != (unsigned char *) NULL);
   assert(chroma2 != (unsigned char *) NULL);
-  buffer=(unsigned char *) AcquireMagickMemory(0x800);
+  buffer=(unsigned char *) AcquireQuantumMemory(0x800,sizeof(*buffer));
   if (buffer == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
@@ -198,7 +193,8 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   {
     PCDGetBits(8);
     length=(sum & 0xff)+1;
-    pcd_table[i]=(PCDTable *) AcquireMagickMemory(length*sizeof(**pcd_table));
+    pcd_table[i]=(PCDTable *) AcquireQuantumMemory(length,
+      sizeof(*pcd_table[i]));
     if (pcd_table[i] == (PCDTable *) NULL)
       {
         buffer=(unsigned char *) RelinquishMagickMemory(buffer);
@@ -206,23 +202,23 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           image->filename);
       }
     r=pcd_table[i];
-    for (j=0; j < (long) length; j++)
+    for (j=0; j < (ssize_t) length; j++)
     {
       PCDGetBits(8);
-      r->length=(sum & 0xff)+1;
+      r->length=(unsigned int) (sum & 0xff)+1;
       if (r->length > 16)
         {
           buffer=(unsigned char *) RelinquishMagickMemory(buffer);
           return(MagickFalse);
         }
       PCDGetBits(16);
-      r->sequence=(sum & 0xffff) << 16;
+      r->sequence=(unsigned int) (sum & 0xffff) << 16;
       PCDGetBits(8);
       r->key=(unsigned char) (sum & 0xff);
       r->mask=(~((1U << (32-r->length))-1));
       r++;
     }
-    pcd_length[i]=length;
+    pcd_length[i]=(size_t) length;
   }
   /*
     Search for Sync byte.
@@ -231,9 +227,9 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     PCDGetBits(16);
   for (i=0; i < 1; i++)
     PCDGetBits(16);
-  while ((sum & 0x00fff000) != 0x00fff000)
+  while ((sum & 0x00fff000UL) != 0x00fff000UL)
     PCDGetBits(8);
-  while (IsSync == 0)
+  while (IsSync(sum) == 0)
     PCDGetBits(1);
   /*
     Recover the Huffman encoded luminance and chrominance deltas.
@@ -245,7 +241,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   q=luma;
   for ( ; ; )
   {
-    if (IsSync != 0)
+    if (IsSync(sum) != 0)
       {
         /*
           Determine plane and row number.
@@ -262,20 +258,20 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           case 0:
           {
             q=luma+row*image->columns;
-            count=(long) image->columns;
+            count=(ssize_t) image->columns;
             break;
           }
           case 2:
           {
             q=chroma1+(row >> 1)*image->columns;
-            count=(long) (image->columns >> 1);
+            count=(ssize_t) (image->columns >> 1);
             plane--;
             break;
           }
           case 3:
           {
             q=chroma2+(row >> 1)*image->columns;
-            count=(long) (image->columns >> 1);
+            count=(ssize_t) (image->columns >> 1);
             plane--;
             break;
           }
@@ -286,44 +282,35 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           }
         }
         length=pcd_length[plane];
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(row,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(DecodeImageTag,row,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
         continue;
       }
     /*
       Decode luminance or chrominance deltas.
     */
     r=pcd_table[plane];
-    for (i=0; ((i < (long) length) && ((sum & r->mask) != r->sequence)); i++)
+    for (i=0; ((i < (ssize_t) length) && ((sum & r->mask) != r->sequence)); i++)
       r++;
-    if ((row > image->rows) || (r == (PCDTable *) NULL) ||
-        ((size_t) (q-luma) > (image->columns*image->rows)))
+    if ((row > image->rows) || (r == (PCDTable *) NULL))
       {
         (void) ThrowMagickException(&image->exception,GetMagickModule(),
           CorruptImageWarning,"SkipToSyncByte","`%s'",image->filename);
         while ((sum & 0x00fff000) != 0x00fff000)
           PCDGetBits(8);
-        while (IsSync == 0)
+        while (IsSync(sum) == 0)
           PCDGetBits(1);
         continue;
       }
     if (r->key < 128)
-      quantum=(long) (*q)+r->key;
+      quantum=(ssize_t) (*q)+r->key;
     else
-      quantum=(long) (*q)+r->key-256;
+      quantum=(ssize_t) (*q)+r->key-256;
     *q=(unsigned char) ((quantum < 0) ? 0 : (quantum > 255) ? 255 : quantum);
     q++;
     PCDGetBits(r->length);
     count--;
   }
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (i=0; i < (image->columns > 1536 ? 3 : 1); i++)
     pcd_table[i]=(PCDTable *) RelinquishMagickMemory(pcd_table[i]);
@@ -351,18 +338,16 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 {
   if (length < 2052)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick+2048,"PCD_",4) == 0)
+  if (LocaleNCompare((const char *) magick+2048,"PCD_",4) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -389,44 +374,43 @@ static MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   ExceptionInfo *exception)
 {
-  const char
-    *commands[3];
-
   Image
     *montage_image;
 
-  ImageInfo
-    *mogrify_info;
-
   MontageInfo
     *montage_info;
 
-  /*
-    Create image tiles.
-  */
-  mogrify_info=CloneImageInfo(image_info);
-  commands[0]=GetClientName();
-  commands[1]="-label";
-  commands[2]=DefaultTileLabel;
-  (void) MogrifyImages(mogrify_info,3,commands,&image,exception);
-  mogrify_info=DestroyImageInfo(mogrify_info);
+  register Image
+    *p;
+
   /*
     Create the PCD Overview image.
   */
+  for (p=image; p != (Image *) NULL; p=p->next)
+  {
+    (void) DeleteImageProperty(p,"label");
+    (void) SetImageProperty(p,"label",DefaultTileLabel);
+  }
   montage_info=CloneMontageInfo(image_info,(MontageInfo *) NULL);
   (void) CopyMagickString(montage_info->filename,image_info->filename,
     MaxTextExtent);
-  montage_image=MontageImages(image,montage_info,exception);
+  montage_image=MontageImageList(image_info,montage_info,image,exception);
   montage_info=DestroyMontageInfo(montage_info);
   if (montage_image == (Image *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -434,14 +418,63 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   return(montage_image);
 }
 
+static void Upsample(const size_t width,const size_t height,
+  const size_t scaled_width,unsigned char *pixels)
+{
+  register ssize_t
+    x,
+    y;
+
+  register unsigned char
+    *p,
+    *q,
+    *r;
+
+  /*
+    Create a new image that is a integral size greater than an existing one.
+  */
+  assert(pixels != (unsigned char *) NULL);
+  for (y=0; y < (ssize_t) height; y++)
+  {
+    p=pixels+(height-1-y)*scaled_width+(width-1);
+    q=pixels+((height-1-y) << 1)*scaled_width+((width-1) << 1);
+    *q=(*p);
+    *(q+1)=(*(p));
+    for (x=1; x < (ssize_t) width; x++)
+    {
+      p--;
+      q-=2;
+      *q=(*p);
+      *(q+1)=(unsigned char) ((((size_t) *p)+((size_t) *(p+1))+1) >> 1);
+    }
+  }
+  for (y=0; y < (ssize_t) (height-1); y++)
+  {
+    p=pixels+((size_t) y << 1)*scaled_width;
+    q=p+scaled_width;
+    r=q+scaled_width;
+    for (x=0; x < (ssize_t) (width-1); x++)
+    {
+      *q=(unsigned char) ((((size_t) *p)+((size_t) *r)+1) >> 1);
+      *(q+1)=(unsigned char) ((((size_t) *p)+((size_t) *(p+2))+
+        ((size_t) *r)+((size_t) *(r+2))+2) >> 2);
+      q+=2;
+      p+=2;
+      r+=2;
+    }
+    *q++=(unsigned char) ((((size_t) *p++)+((size_t) *r++)+1) >> 1);
+    *q++=(unsigned char) ((((size_t) *p++)+((size_t) *r++)+1) >> 1);
+  }
+  p=pixels+(2*height-2)*scaled_width;
+  q=pixels+(2*height-1)*scaled_width;
+  (void) CopyMagickMemory(q,p,(size_t) (2*width));
+}
+
 static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
     *image;
 
-  long
-    x;
-
   MagickBooleanType
     status;
 
@@ -451,7 +484,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickSizeType
     number_pixels;
 
-  register long
+  register ssize_t
     i,
     y;
 
@@ -464,7 +497,15 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *yy;
 
   size_t
-    count;
+    height,
+    number_images,
+    rotate,
+    scene,
+    width;
+
+  ssize_t
+    count,
+    x;
 
   unsigned char
     *chroma1,
@@ -475,13 +516,6 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned int
     overview;
 
-  unsigned long
-    height,
-    number_images,
-    rotate,
-    scene,
-    width;
-
   /*
     Open image file.
   */
@@ -492,7 +526,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -500,15 +534,15 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   /*
-    Determine if this is a PCD file.
+    Determine if this a PCD file.
   */
-  header=(unsigned char *) AcquireMagickMemory(3*0x800);
+  header=(unsigned char *) AcquireQuantumMemory(0x800,3UL*sizeof(*header));
   if (header == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   count=ReadBlob(image,3*0x800,header);
   overview=LocaleNCompare((char *) header,"PCD_OPA",7) == 0;
   if ((count == 0) ||
-      ((LocaleNCompare((char *) header+0x800,"PCD",3) != 0) && !overview))
+      ((LocaleNCompare((char *) header+0x800,"PCD",3) != 0) && (overview == 0)))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   rotate=header[0x0e02] & 0x03;
   number_images=(header[10] << 8) | header[11];
@@ -531,15 +565,15 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     }
   if (image_info->number_scenes != 0)
-    scene=Min(image_info->scene,6);
-  if (overview)
+    scene=(size_t) MagickMin(image_info->scene,6);
+  if (overview != 0)
     scene=1;
   /*
     Initialize image structure.
   */
   width=192;
   height=128;
-  for (i=1; i < (long) Min(scene,3); i++)
+  for (i=1; i < (ssize_t) MagickMin(scene,3); i++)
   {
     width<<=1;
     height<<=1;
@@ -547,7 +581,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->columns=width;
   image->rows=height;
   image->depth=8;
-  for ( ; i < (long) scene; i++)
+  for ( ; i < (ssize_t) scene; i++)
   {
     image->columns<<=1;
     image->rows<<=1;
@@ -558,9 +592,12 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   number_pixels=(MagickSizeType) image->columns*image->rows;
   if (number_pixels != (size_t) number_pixels)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  chroma1=(unsigned char *) AcquireMagickMemory(image->columns*image->rows+1);
-  chroma2=(unsigned char *) AcquireMagickMemory(image->columns*image->rows+1);
-  luma=(unsigned char *) AcquireMagickMemory(image->columns*image->rows+1);
+  chroma1=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
+    sizeof(*chroma1));
+  chroma2=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
+    sizeof(*chroma2));
+  luma=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
+    sizeof(*luma));
   if ((chroma1 == (unsigned char *) NULL) ||
       (chroma2 == (unsigned char *) NULL) || (luma == (unsigned char *) NULL))
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -568,7 +605,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Advance to image data.
   */
   offset=93;
-  if (overview)
+  if (overview != 0)
     offset=2;
   else
     if (scene == 2)
@@ -576,9 +613,9 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     else
       if (scene <= 1)
         offset=1;
-  for (i=0; i < (long) (offset*0x800); i++)
+  for (i=0; i < (ssize_t) (offset*0x800); i++)
     (void) ReadBlobByte(image);
-  if (overview)
+  if (overview != 0)
     {
       Image
         *overview_image;
@@ -586,28 +623,28 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       MagickProgressMonitor
         progress_monitor;
 
-      register long
+      register ssize_t
         j;
 
       /*
         Read thumbnails from overview image.
       */
-      for (j=1; j <= (long) number_images; j++)
+      for (j=1; j <= (ssize_t) number_images; j++)
       {
         progress_monitor=SetImageProgressMonitor(image,
           (MagickProgressMonitor) NULL,image->client_data);
-        (void) FormatMagickString(image->filename,MaxTextExtent,
-          "images/img%04ld.pcd",j);
-        (void) FormatMagickString(image->magick_filename,MaxTextExtent,
-          "images/img%04ld.pcd",j);
-        image->scene=j;
+        (void) FormatLocaleString(image->filename,MaxTextExtent,
+          "images/img%04ld.pcd",(long) j);
+        (void) FormatLocaleString(image->magick_filename,MaxTextExtent,
+          "images/img%04ld.pcd",(long) j);
+        image->scene=(size_t) j;
         image->columns=width;
         image->rows=height;
         image->depth=8;
         yy=luma;
         c1=chroma1;
         c2=chroma2;
-        for (y=0; y < (long) height; y+=2)
+        for (y=0; y < (ssize_t) height; y+=2)
         {
           count=ReadBlob(image,width,yy);
           yy+=image->columns;
@@ -626,30 +663,30 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         yy=luma;
         c1=chroma1;
         c2=chroma2;
-        for (y=0; y < (long) image->rows; y++)
+        for (y=0; y < (ssize_t) image->rows; y++)
         {
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
-            q->red=ScaleCharToQuantum(*yy++);
-            q->green=ScaleCharToQuantum(*c1++);
-            q->blue=ScaleCharToQuantum(*c2++);
+            SetPixelRed(q,ScaleCharToQuantum(*yy++));
+            SetPixelGreen(q,ScaleCharToQuantum(*c1++));
+            SetPixelBlue(q,ScaleCharToQuantum(*c2++));
             q++;
           }
-          if (SyncImagePixels(image) == MagickFalse)
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
         }
         image->colorspace=YCCColorspace;
         if (LocaleCompare(image_info->magick,"PCDS") == 0)
           image->colorspace=sRGBColorspace;
-        if (j < (long) number_images)
+        if (j < (ssize_t) number_images)
           {
             /*
               Allocate next image structure.
             */
-            AllocateNextImage(image_info,image);
+            AcquireNextImage(image_info,image);
             if (GetNextImageInList(image) == (Image *) NULL)
               {
                 image=DestroyImageList(image);
@@ -659,11 +696,9 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
         (void) SetImageProgressMonitor(image,progress_monitor,
           image->client_data);
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(j-1,number_images) != MagickFalse))
+        if (image->previous == (Image *) NULL)
           {
-            status=image->progress_monitor(LoadImageTag,j-1,number_images,
-              image->client_data);
+            status=SetImageProgress(image,LoadImageTag,j-1,number_images);
             if (status == MagickFalse)
               break;
           }
@@ -681,7 +716,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   yy=luma;
   c1=chroma1;
   c2=chroma2;
-  for (y=0; y < (long) height; y+=2)
+  for (y=0; y < (ssize_t) height; y+=2)
   {
     count=ReadBlob(image,width,yy);
     yy+=image->columns;
@@ -714,7 +749,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Upsample(768,512,image->columns,chroma2);
           image->rows=2048;
           offset=TellBlob(image)/0x800+12;
-          (void) SeekBlob(image,offset*0x800,SEEK_SET);
+          offset=SeekBlob(image,offset*0x800,SEEK_SET);
           status=DecodeImage(image,luma,chroma1,chroma2);
           if ((scene >= 6) && (status != MagickFalse))
             {
@@ -736,25 +771,24 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   yy=luma;
   c1=chroma1;
   c2=chroma2;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      q->red=ScaleCharToQuantum(*yy++);
-      q->green=ScaleCharToQuantum(*c1++);
-      q->blue=ScaleCharToQuantum(*c2++);
+      SetPixelRed(q,ScaleCharToQuantum(*yy++));
+      SetPixelGreen(q,ScaleCharToQuantum(*c1++));
+      SetPixelBlue(q,ScaleCharToQuantum(*c2++));
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (image->previous == (Image *) NULL)
       {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
+        status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                image->rows);
         if (status == MagickFalse)
           break;
       }
@@ -765,7 +799,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   if (image_info->ping == MagickFalse)
     if ((rotate == 1) || (rotate == 3))
       {
@@ -824,29 +858,32 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterPCDImage method is:
 %
-%      RegisterPCDImage(void)
+%      size_t RegisterPCDImage(void)
 %
 */
-ModuleExport void RegisterPCDImage(void)
+ModuleExport size_t RegisterPCDImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("PCD");
-  entry->decoder=(DecoderHandler *) ReadPCDImage;
-  entry->encoder=(EncoderHandler *) WritePCDImage;
-  entry->magick=(MagickHandler *) IsPCD;
+  entry->decoder=(DecodeImageHandler *) ReadPCDImage;
+  entry->encoder=(EncodeImageHandler *) WritePCDImage;
+  entry->magick=(IsImageFormatHandler *) IsPCD;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Photo CD");
-  entry->module=AcquireString("PCD");
+  entry->seekable_stream=MagickTrue;
+  entry->description=ConstantString("Photo CD");
+  entry->module=ConstantString("PCD");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PCDS");
-  entry->decoder=(DecoderHandler *) ReadPCDImage;
-  entry->encoder=(EncoderHandler *) WritePCDImage;
+  entry->decoder=(DecodeImageHandler *) ReadPCDImage;
+  entry->encoder=(EncodeImageHandler *) WritePCDImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Photo CD");
-  entry->module=AcquireString("PCD");
+  entry->seekable_stream=MagickTrue;
+  entry->description=ConstantString("Photo CD");
+  entry->module=ConstantString("PCD");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -885,8 +922,7 @@ ModuleExport void UnregisterPCDImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  WritePCDImage() writes an image in the Photo CD encoded image
-%  format.
+%  WritePCDImage() writes an image in the Photo CD encoded image format.
 %
 %  The format of the WritePCDImage method is:
 %
@@ -894,15 +930,14 @@ ModuleExport void UnregisterPCDImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
 
-static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
-  const char *page_geometry,const char *tile_geometry)
+static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
+  const char *tile_geometry)
 {
   GeometryInfo
     geometry_info;
@@ -911,9 +946,6 @@ static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
     *downsample_image,
     *tile_image;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -927,10 +959,13 @@ static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
     *p,
     *q;
 
-  register long
+  register ssize_t
     i,
     x;
 
+  ssize_t
+    y;
+
   /*
     Scale image to tile size.
   */
@@ -946,8 +981,8 @@ static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
   if (tile_image == (Image *) NULL)
     return(MagickFalse);
   flags=ParseGeometry(page_geometry,&geometry_info);
-  geometry.width=(unsigned long) geometry_info.rho;
-  geometry.height=(unsigned long) geometry_info.sigma;
+  geometry.width=(size_t) geometry_info.rho;
+  geometry.height=(size_t) geometry_info.sigma;
   if ((flags & SigmaValue) == 0)
     geometry.height=geometry.width;
   if ((tile_image->columns != geometry.width) ||
@@ -971,7 +1006,8 @@ static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
       tile_image=bordered_image;
     }
   (void) TransformImage(&tile_image,(char *) NULL,tile_geometry);
-  (void) SetImageColorspace(tile_image,YCCColorspace);
+  if (IsRGBColorspace(image->colorspace) == MagickFalse)
+    (void) TransformImageColorspace(tile_image,YCCColorspace);
   downsample_image=ResizeImage(tile_image,tile_image->columns/2,
     tile_image->rows/2,TriangleFilter,1.0,&image->exception);
   if (downsample_image == (Image *) NULL)
@@ -979,43 +1015,38 @@ static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
   /*
     Write tile to PCD file.
   */
-  for (y=0; y < (long) tile_image->rows; y+=2)
+  for (y=0; y < (ssize_t) tile_image->rows; y+=2)
   {
-    p=AcquireImagePixels(tile_image,0,y,tile_image->columns,2,
+    p=GetVirtualPixels(tile_image,0,y,tile_image->columns,2,
       &tile_image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) (tile_image->columns << 1); x++)
+    for (x=0; x < (ssize_t) (tile_image->columns << 1); x++)
     {
-      (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
+      (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelRed(p)));
       p++;
     }
-    q=AcquireImagePixels(downsample_image,0,y >> 1,downsample_image->columns,
+    q=GetVirtualPixels(downsample_image,0,y >> 1,downsample_image->columns,
       1,&downsample_image->exception);
     if (q == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) downsample_image->columns; x++)
+    for (x=0; x < (ssize_t) downsample_image->columns; x++)
     {
-      (void) WriteBlobByte(image,ScaleQuantumToChar(q->green));
+      (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelGreen(q)));
       q++;
     }
-    q=AcquireImagePixels(downsample_image,0,y >> 1,downsample_image->columns,
+    q=GetVirtualPixels(downsample_image,0,y >> 1,downsample_image->columns,
       1,&downsample_image->exception);
     if (q == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) downsample_image->columns; x++)
+    for (x=0; x < (ssize_t) downsample_image->columns; x++)
     {
-      (void) WriteBlobByte(image,ScaleQuantumToChar(q->blue));
+      (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelBlue(q)));
       q++;
     }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,tile_image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,tile_image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    status=SetImageProgress(image,SaveImageTag,y,tile_image->rows);
+    if (status == MagickFalse)
+      break;
   }
   for (i=0; i < 0x800; i++)
     (void) WriteBlobByte(image,'\0');
@@ -1032,7 +1063,7 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register long
+  register ssize_t
     i;
 
   assert(image_info != (const ImageInfo *) NULL);
@@ -1061,7 +1092,8 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,pcd_image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(pcd_image,RGBColorspace);
+  if (IsRGBColorspace(image->colorspace) == MagickFalse)
+    (void) TransformImageColorspace(pcd_image,RGBColorspace);
   /*
     Write PCD image header.
   */
@@ -1085,7 +1117,7 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(pcd_image,0x01);
   for (i=0; i < 1944; i++)
     (void) WriteBlobByte(pcd_image,'\0');
-  (void) WriteBlob(pcd_image,7,(unsigned char *) "PCD_IPI");
+  (void) WriteBlob(pcd_image,7,(const unsigned char *) "PCD_IPI");
   (void) WriteBlobByte(pcd_image,0x06);
   for (i=0; i < 1530; i++)
     (void) WriteBlobByte(pcd_image,'\0');
@@ -1098,10 +1130,10 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   /*
     Write PCD tiles.
   */
-  status=WritePCDTile(image_info,pcd_image,"768x512>","192x128");
-  status=WritePCDTile(image_info,pcd_image,"768x512>","384x256");
-  status=WritePCDTile(image_info,pcd_image,"768x512>","768x512");
-  CloseBlob(pcd_image);
+  status=WritePCDTile(pcd_image,"768x512>","192x128");
+  status=WritePCDTile(pcd_image,"768x512>","384x256");
+  status=WritePCDTile(pcd_image,"768x512>","768x512");
+  (void) CloseBlob(pcd_image);
   if (pcd_image != image)
     pcd_image=DestroyImage(pcd_image);
   return(status);
