@@ -10,14 +10,14 @@
 %                            P      IIIII  X   X                              %
 %                                                                             %
 %                                                                             %
-%                    Read Alias/Wavefront RLE Image Format.                   %
+%                    Read Alias/Wavefront RLE Image Format                    %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,8 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/colormap.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
@@ -50,8 +52,11 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -74,7 +79,7 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -88,9 +93,6 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   IndexPacket
     index;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -102,20 +104,21 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register IndexPacket
     *indexes;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
     *q;
 
   size_t
-    length;
-
-  unsigned long
     bits_per_pixel,
     height,
+    length,
     width;
 
+  ssize_t
+    y;
+
   /*
     Open image file.
   */
@@ -126,7 +129,7 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -141,8 +144,8 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) ReadBlobMSBShort(image);  /* x-offset */
   (void) ReadBlobMSBShort(image);  /* y-offset */
   bits_per_pixel=ReadBlobMSBShort(image);
-  if ((width == (unsigned long) ~0) || (height == (unsigned long) ~0) ||
-      ((bits_per_pixel != 8) && (bits_per_pixel != 24)))
+  if ((width == 0UL) || (height == 0UL) || ((bits_per_pixel != 8) &&
+      (bits_per_pixel != 24)))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   do
   {
@@ -152,7 +155,7 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->columns=width;
     image->rows=height;
     if (bits_per_pixel == 8)
-      if (AllocateImageColormap(image,256) == MagickFalse)
+      if (AcquireImageColormap(image,256) == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
@@ -160,50 +163,48 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Convert PIX raster image to pixel packets.
     */
-    red=0;
-    green=0;
-    blue=0;
-    index=0;
+    red=(Quantum) 0;
+    green=(Quantum) 0;
+    blue=(Quantum) 0;
+    index=(IndexPacket) 0;
     length=0;
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
-      q=SetImagePixels(image,0,y,image->columns,1);
+      q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
-      for (x=0; x < (long) image->columns; x++)
+      indexes=GetAuthenticIndexQueue(image);
+      for (x=0; x < (ssize_t) image->columns; x++)
       {
         if (length == 0)
           {
             length=(size_t) ReadBlobByte(image);
             if (bits_per_pixel == 8)
-              index=ScaleCharToQuantum(ReadBlobByte(image));
+              index=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             else
               {
-                blue=ScaleCharToQuantum(ReadBlobByte(image));
-                green=ScaleCharToQuantum(ReadBlobByte(image));
-                red=ScaleCharToQuantum(ReadBlobByte(image));
+                blue=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+                green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+                red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
               }
           }
         if (image->storage_class == PseudoClass)
-          indexes[x]=index;
-        q->blue=blue;
-        q->green=green;
-        q->red=red;
+          SetPixelIndex(indexes+x,index);
+        SetPixelBlue(q,blue);
+        SetPixelGreen(q,green);
+        SetPixelRed(q,red);
         length--;
         q++;
       }
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(LoadImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        {
+          status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+            image->rows);
+          if (status == MagickFalse)
+            break;
+        }
     }
     if (image->storage_class == PseudoClass)
       (void) SyncImage(image);
@@ -224,31 +225,27 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ReadBlobMSBShort(image);
     (void) ReadBlobMSBShort(image);
     bits_per_pixel=ReadBlobMSBShort(image);
-    status=(MagickBooleanType) ((width != (unsigned long) ~0) &&
-      (height == (unsigned long) ~0) && ((bits_per_pixel == 8) ||
-      (bits_per_pixel == 24)));
+    status=(width != 0UL) && (height == 0UL) && ((bits_per_pixel == 8) ||
+      (bits_per_pixel == 24)) ? MagickTrue : MagickFalse;
     if (status == MagickTrue)
       {
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   } while (status == MagickTrue);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -272,19 +269,20 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterPIXImage method is:
 %
-%      RegisterPIXImage(void)
+%      size_t RegisterPIXImage(void)
 %
 */
-ModuleExport void RegisterPIXImage(void)
+ModuleExport size_t RegisterPIXImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("PIX");
-  entry->decoder=(DecoderHandler *) ReadPIXImage;
-  entry->description=AcquireString("Alias/Wavefront RLE image format");
-  entry->module=AcquireString("PIX");
+  entry->decoder=(DecodeImageHandler *) ReadPIXImage;
+  entry->description=ConstantString("Alias/Wavefront RLE image format");
+  entry->module=ConstantString("PIX");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
