@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -100,9 +100,6 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
     *canvas_image,
     *image;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
@@ -115,15 +112,13 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register long
-    i;
-
-  ssize_t
-    count;
-
   size_t
     length;
 
+  ssize_t
+    count,
+    y;
+
   unsigned char
     *pixels;
 
@@ -146,13 +141,9 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  for (i=0; i < image->offset; i++)
-    if (ReadBlobByte(image) == EOF)
-      {
-        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
-          image->filename);
-        break;
-      }
+  if (DiscardBlobBytes(image,image->offset) == MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   /*
     Create virtual canvas to support cropping (i.e. image.gray[100x100+10+20]).
   */
@@ -172,7 +163,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
       */
       image->scene++;
       length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
-      for (y=0; y < (long) image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
         count=ReadBlob(image,length,pixels);
         if (count != (ssize_t) length)
@@ -195,16 +186,16 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
         length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
         count=ReadBlob(image,length,pixels);
       }
-    for (y=0; y < (long) image->extract_info.height; y++)
+    for (y=0; y < (ssize_t) image->extract_info.height; y++)
     {
       register const PixelPacket
-        *__restrict p;
-
-      register long
-        x;
+        *restrict p;
 
       register PixelPacket
-        *__restrict q;
+        *restrict q;
+
+      register ssize_t
+        x;
 
       if (count != (ssize_t) length)
         {
@@ -220,7 +211,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
       if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
         break;
       if (((y-image->extract_info.y) >= 0) && 
-          ((y-image->extract_info.y) < (long) image->rows))
+          ((y-image->extract_info.y) < (ssize_t) image->rows))
         {
           p=GetVirtualPixels(canvas_image,canvas_image->extract_info.x,0,
             image->columns,1,exception);
@@ -228,11 +219,11 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
             1,exception);
           if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
             break;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
-            q->red=p->red;
-            q->green=p->green;
-            q->blue=p->blue;
+            SetPixelRed(q,GetPixelRed(p));
+            SetPixelGreen(q,GetPixelGreen(p));
+            SetPixelBlue(q,GetPixelBlue(p));
             p++;
             q++;
           }
@@ -241,7 +232,8 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
         }
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
@@ -299,10 +291,10 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterRAWImage method is:
 %
-%      unsigned long RegisterRAWImage(void)
+%      size_t RegisterRAWImage(void)
 %
 */
-ModuleExport unsigned long RegisterRAWImage(void)
+ModuleExport size_t RegisterRAWImage(void)
 {
   MagickInfo
     *entry;
@@ -312,7 +304,6 @@ ModuleExport unsigned long RegisterRAWImage(void)
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw red samples");
   entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -321,7 +312,6 @@ ModuleExport unsigned long RegisterRAWImage(void)
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw cyan samples");
   entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -330,7 +320,6 @@ ModuleExport unsigned long RegisterRAWImage(void)
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw green samples");
   entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -339,7 +328,6 @@ ModuleExport unsigned long RegisterRAWImage(void)
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw magenta samples");
   entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -349,7 +337,6 @@ ModuleExport unsigned long RegisterRAWImage(void)
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
   entry->description=ConstantString("Raw blue samples");
-  entry->format_type=ExplicitFormatType;
   entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("Y");
@@ -357,7 +344,6 @@ ModuleExport unsigned long RegisterRAWImage(void)
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw yellow samples");
   entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -374,7 +360,6 @@ ModuleExport unsigned long RegisterRAWImage(void)
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw opacity samples");
   entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -383,7 +368,6 @@ ModuleExport unsigned long RegisterRAWImage(void)
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw black samples");
   entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -448,9 +432,6 @@ ModuleExport void UnregisterRAWImage(void)
 */
 static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
 {
-  long
-    y;
-
   MagickOffsetType
     scene;
 
@@ -466,12 +447,13 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  ssize_t
-    count;
-
   size_t
     length;
 
+  ssize_t
+    count,
+    y;
+
   unsigned char
     *pixels;
 
@@ -573,7 +555,7 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
     if (quantum_info == (QuantumInfo *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     pixels=GetQuantumPixels(quantum_info);
-    for (y=0; y < (long) image->rows; y++)
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
       p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
@@ -585,7 +567,8 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
         break;
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,SaveImageTag,y,image->rows);
+          status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
