@@ -10,20 +10,20 @@
 %                            R  R   LLLLL  EEEEE                              %
 %                                                                             %
 %                                                                             %
-%                          Read URT RLE Image Format.                         %
+%                          Read URT RLE Image Format                          %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -40,16 +40,25 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
+#include "magick/property.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/colormap.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/quantum-private.h"
+#include "magick/pixel.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -71,10 +80,7 @@
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsRLE() returns MagickTrue if the image format type is RLE.
-%
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -110,11 +116,7 @@ static MagickBooleanType IsRLE(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadRLEImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -140,9 +142,6 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
     operand,
     status;
 
-  long
-    y;
-
   MagickStatusType
     flags;
 
@@ -152,20 +151,28 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register IndexPacket
     *indexes;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
     *q;
 
-  register long
+  register ssize_t
     i;
 
   register unsigned char
     *p;
 
+  size_t
+    bits_per_pixel,
+    map_length,
+    number_colormaps,
+    number_planes,
+    one;
+
   ssize_t
-    count;
+    count,
+    y;
 
   unsigned char
     background_color[256],
@@ -174,30 +181,25 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
     plane,
     *rle_pixels;
 
-  unsigned long
-    bits_per_pixel,
-    map_length,
-    number_colormaps,
-    number_planes;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
-    Determine if this is a RLE file.
+    Determine if this a RLE file.
   */
   count=ReadBlob(image,2,(unsigned char *) magick);
   if ((count == 0) || (memcmp(magick,"\122\314",2) != 0))
@@ -211,21 +213,22 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ReadBlobLSBShort(image);
     image->columns=ReadBlobLSBShort(image);
     image->rows=ReadBlobLSBShort(image);
-    flags=ReadBlobByte(image);
-    image->matte=flags & 0x04;
-    number_planes=ReadBlobByte(image);
-    bits_per_pixel=ReadBlobByte(image);
-    number_colormaps=ReadBlobByte(image);
-    map_length=1 << ReadBlobByte(image);
+    flags=(MagickStatusType) ReadBlobByte(image);
+    image->matte=flags & 0x04 ? MagickTrue : MagickFalse;
+    number_planes=1UL*ReadBlobByte(image);
+    bits_per_pixel=1UL*ReadBlobByte(image);
+    number_colormaps=1UL*ReadBlobByte(image);
+    one=1;
+    map_length=one << ReadBlobByte(image);
     if ((number_planes == 0) || (number_planes == 2) || (bits_per_pixel != 8) ||
         (image->columns == 0))
-      ThrowReaderException(CorruptImageError,"UnsupportedRLEImageFile");
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     if (flags & 0x02)
       {
         /*
           No background color-- initialize to black.
         */
-        for (i=0; i < (long) number_planes; i++)
+        for (i=0; i < (ssize_t) number_planes; i++)
           background_color[i]=0;
         (void) ReadBlobByte(image);
       }
@@ -235,8 +238,8 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Initialize background color.
         */
         p=background_color;
-        for (i=0; i < (long) number_planes; i++)
-          *p++=ReadBlobByte(image);
+        for (i=0; i < (ssize_t) number_planes; i++)
+          *p++=(unsigned char) ReadBlobByte(image);
       }
     if ((number_planes & 0x01) == 0)
       (void) ReadBlobByte(image);
@@ -246,21 +249,21 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read image colormaps.
         */
-        colormap=(unsigned char *)
-          AcquireMagickMemory(number_colormaps*map_length);
+        colormap=(unsigned char *) AcquireQuantumMemory(number_colormaps,
+          map_length*sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         p=colormap;
-        for (i=0; i < (long) number_colormaps; i++)
-          for (x=0; x < (long) map_length; x++)
-            *p++=ScaleShortToQuantum(ReadBlobLSBShort(image));
+        for (i=0; i < (ssize_t) number_colormaps; i++)
+          for (x=0; x < (ssize_t) map_length; x++)
+            *p++=(unsigned char) ScaleShortToQuantum(ReadBlobLSBShort(image));
       }
     if ((flags & 0x08) != 0)
       {
         char
           *comment;
 
-        unsigned long
+        size_t
           length;
 
         /*
@@ -269,13 +272,13 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         length=ReadBlobLSBShort(image);
         if (length != 0)
           {
-            comment=(char *) AcquireMagickMemory(length);
+            comment=(char *) AcquireQuantumMemory(length,sizeof(*comment));
             if (comment == (char *) NULL)
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-            (void) ReadBlob(image,length-1,(unsigned char *) comment);
+            count=ReadBlob(image,length-1,(unsigned char *) comment);
             comment[length-1]='\0';
-            (void) SetImageAttribute(image,"comment",comment);
-            comment=(char *) RelinquishMagickMemory(comment);
+            (void) SetImageProperty(image,"comment",comment);
+            comment=DestroyString(comment);
             if ((length & 0x01) == 0)
               (void) ReadBlobByte(image);
           }
@@ -291,27 +294,27 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
     number_pixels=(MagickSizeType) image->columns*image->rows;
     if ((number_pixels*number_planes) != (size_t) (number_pixels*number_planes))
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    rle_pixels=(unsigned char *)
-      AcquireMagickMemory(image->columns*image->rows*number_planes);
+    rle_pixels=(unsigned char *) AcquireQuantumMemory(image->columns,
+      image->rows*number_planes*sizeof(*rle_pixels));
     if (rle_pixels == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((flags & 0x01) && !(flags & 0x02))
       {
-        long
+        ssize_t
           j;
 
         /*
           Set background color.
         */
         p=rle_pixels;
-        for (i=0; i < (long) number_pixels; i++)
+        for (i=0; i < (ssize_t) number_pixels; i++)
         {
           if (image->matte == MagickFalse)
-            for (j=0; j < (long) number_planes; j++)
+            for (j=0; j < (ssize_t) number_planes; j++)
               *p++=background_color[j];
           else
             {
-              for (j=0; j < (long) (number_planes-1); j++)
+              for (j=0; j < (ssize_t) (number_planes-1); j++)
                 *p++=background_color[j];
               *p++=0;  /* initialize matte channel */
             }
@@ -332,7 +335,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           operand=ReadBlobByte(image);
           if (opcode & 0x40)
-            operand=ReadBlobLSBShort(image);
+            operand=(int) ReadBlobLSBShort(image);
           x=0;
           y+=operand;
           break;
@@ -350,7 +353,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           operand=ReadBlobByte(image);
           if (opcode & 0x40)
-            operand=ReadBlobLSBShort(image);
+            operand=(int) ReadBlobLSBShort(image);
           x+=operand;
           break;
         }
@@ -358,14 +361,15 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           operand=ReadBlobByte(image);
           if (opcode & 0x40)
-            operand=ReadBlobLSBShort(image);
+            operand=(int) ReadBlobLSBShort(image);
           p=rle_pixels+((image->rows-y-1)*image->columns*number_planes)+
             x*number_planes+plane;
           operand++;
-          for (i=0; i < (long) operand; i++)
+          for (i=0; i < (ssize_t) operand; i++)
           {
-            pixel=ReadBlobByte(image);
-            if ((y < (long) image->rows) && ((x+i) < (long) image->columns))
+            pixel=(unsigned char) ReadBlobByte(image);
+            if ((y < (ssize_t) image->rows) &&
+                ((x+i) < (ssize_t) image->columns))
               *p=pixel;
             p+=number_planes;
           }
@@ -378,15 +382,16 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           operand=ReadBlobByte(image);
           if (opcode & 0x40)
-            operand=ReadBlobLSBShort(image);
-          pixel=ReadBlobByte(image);
+            operand=(int) ReadBlobLSBShort(image);
+          pixel=(unsigned char) ReadBlobByte(image);
           (void) ReadBlobByte(image);
           operand++;
           p=rle_pixels+((image->rows-y-1)*image->columns*number_planes)+
             x*number_planes+plane;
-          for (i=0; i < (long) operand; i++)
+          for (i=0; i < (ssize_t) operand; i++)
           {
-            if ((y < (long) image->rows) && ((x+i) < (long) image->columns))
+            if ((y < (ssize_t) image->rows) &&
+                ((x+i) < (ssize_t) image->columns))
               *p=pixel;
             p+=number_planes;
           }
@@ -406,18 +411,18 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Apply colormap affineation to image.
         */
-        mask=(map_length-1);
+        mask=(MagickStatusType) (map_length-1);
         p=rle_pixels;
         if (number_colormaps == 1)
-          for (i=0; i < (long) number_pixels; i++)
+          for (i=0; i < (ssize_t) number_pixels; i++)
           {
             *p=colormap[*p & mask];
             p++;
           }
         else
           if ((number_planes >= 3) && (number_colormaps >= 3))
-            for (i=0; i < (long) number_pixels; i++)
-              for (x=0; x < (long) number_planes; x++)
+            for (i=0; i < (ssize_t) number_pixels; i++)
+              for (x=0; x < (ssize_t) number_planes; x++)
               {
                 *p=colormap[x*map_length+(*p & mask)];
                 p++;
@@ -432,26 +437,29 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Convert raster image to DirectClass pixel packets.
         */
         p=rle_pixels;
-        for (y=0; y < (long) image->rows; y++)
+        for (y=0; y < (ssize_t) image->rows; y++)
         {
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
-            q->red=ScaleCharToQuantum(*p++);
-            q->green=ScaleCharToQuantum(*p++);
-            q->blue=ScaleCharToQuantum(*p++);
+            SetPixelRed(q,ScaleCharToQuantum(*p++));
+            SetPixelGreen(q,ScaleCharToQuantum(*p++));
+            SetPixelBlue(q,ScaleCharToQuantum(*p++));
             if (image->matte != MagickFalse)
-              q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*p++));
+              SetPixelAlpha(q,ScaleCharToQuantum(*p++));
             q++;
           }
-          if (SyncImagePixels(image) == MagickFalse)
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
+            {
+              status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                image->rows);
+              if (status == MagickFalse)
                 break;
+            }
         }
       }
     else
@@ -461,22 +469,22 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (number_colormaps == 0)
           map_length=256;
-        if (AllocateImageColormap(image,map_length) == MagickFalse)
+        if (AcquireImageColormap(image,map_length) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         p=colormap;
         if (number_colormaps == 1)
-          for (i=0; i < (long) image->colors; i++)
+          for (i=0; i < (ssize_t) image->colors; i++)
           {
             /*
               Pseudocolor.
             */
-            image->colormap[i].red=ScaleCharToQuantum(i);
-            image->colormap[i].green=ScaleCharToQuantum(i);
-            image->colormap[i].blue=ScaleCharToQuantum(i);
+            image->colormap[i].red=ScaleCharToQuantum((unsigned char) i);
+            image->colormap[i].green=ScaleCharToQuantum((unsigned char) i);
+            image->colormap[i].blue=ScaleCharToQuantum((unsigned char) i);
           }
         else
           if (number_colormaps > 1)
-            for (i=0; i < (long) image->colors; i++)
+            for (i=0; i < (ssize_t) image->colors; i++)
             {
               image->colormap[i].red=ScaleCharToQuantum(*p);
               image->colormap[i].green=ScaleCharToQuantum(*(p+map_length));
@@ -489,23 +497,23 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Convert raster image to PseudoClass pixel packets.
             */
-            for (y=0; y < (long) image->rows; y++)
+            for (y=0; y < (ssize_t) image->rows; y++)
             {
-              q=SetImagePixels(image,0,y,image->columns,1);
+              q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
               if (q == (PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(image);
-              for (x=0; x < (long) image->columns; x++)
-                indexes[x]=(*p++);
-              if (SyncImagePixels(image) == MagickFalse)
+              indexes=GetAuthenticIndexQueue(image);
+              for (x=0; x < (ssize_t) image->columns; x++)
+                SetPixelIndex(indexes+x,*p++);
+              if (SyncAuthenticPixels(image,exception) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != 0)
-                  {
-                    status=MagickMonitor(LoadImageTag,y,image->rows,exception);
-                    if (status == MagickFalse)
-                      break;
-                  }
+                {
+                  status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
+                    y,image->rows);
+                  if (status == MagickFalse)
+                    break;
+                }
             }
             (void) SyncImage(image);
           }
@@ -514,31 +522,31 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Image has a matte channel-- promote to DirectClass.
             */
-            for (y=0; y < (long) image->rows; y++)
+            for (y=0; y < (ssize_t) image->rows; y++)
             {
-              q=SetImagePixels(image,0,y,image->columns,1);
+              q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
               if (q == (PixelPacket *) NULL)
                 break;
-              for (x=0; x < (long) image->columns; x++)
+              for (x=0; x < (ssize_t) image->columns; x++)
               {
-                q->red=image->colormap[*p++].red;
-                q->green=image->colormap[*p++].green;
-                q->blue=image->colormap[*p++].blue;
-                q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*p++));
+                SetPixelRed(q,image->colormap[*p++].red);
+                SetPixelGreen(q,image->colormap[*p++].green);
+                SetPixelBlue(q,image->colormap[*p++].blue);
+                SetPixelAlpha(q,ScaleCharToQuantum(*p++));
                 q++;
               }
-              if (SyncImagePixels(image) == MagickFalse)
+              if (SyncAuthenticPixels(image,exception) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != 0)
-                  {
-                    status=MagickMonitor(LoadImageTag,y,image->rows,exception);
-                    if (status == MagickFalse)
-                      break;
-                  }
+                {
+                  status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
+                    y,image->rows);
+                  if (status == MagickFalse)
+                    break;
+                }
             }
-            image->colormap=(PixelPacket *)
-              RelinquishMagickMemory(image->colormap);
+            image->colormap=(PixelPacket *) RelinquishMagickMemory(
+              image->colormap);
             image->storage_class=DirectClass;
             image->colors=0;
           }
@@ -565,20 +573,20 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        AcquireNextImage(image_info,image);
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
         if (status == MagickFalse)
           break;
       }
   } while ((count != 0) && (memcmp(magick,"\122\314",2) == 0));
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -602,21 +610,22 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterRLEImage method is:
 %
-%      RegisterRLEImage(void)
+%      size_t RegisterRLEImage(void)
 %
 */
-ModuleExport void RegisterRLEImage(void)
+ModuleExport size_t RegisterRLEImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("RLE");
-  entry->decoder=(DecoderHandler *) ReadRLEImage;
-  entry->magick=(MagickHandler *) IsRLE;
+  entry->decoder=(DecodeImageHandler *) ReadRLEImage;
+  entry->magick=(IsImageFormatHandler *) IsRLE;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Utah Run length encoded image");
-  entry->module=AcquireString("RLE");
+  entry->description=ConstantString("Utah Run length encoded image");
+  entry->module=ConstantString("RLE");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
