@@ -10,14 +10,14 @@
 %                         UUU     Y      V      Y                             %
 %                                                                             %
 %                                                                             %
-%            Read/Write 16bit/pixel Interleaved YUV Image Format.             %
+%            Read/Write 16bit/pixel Interleaved YUV Image Format              %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
@@ -52,8 +53,11 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 
 /*
   Forward declarations.
@@ -83,11 +87,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadUYVYImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
@@ -95,20 +98,17 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
   Image
     *image;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
     *q;
 
-  register long
-    i;
+  ssize_t
+    y;
 
   unsigned char
     u,
@@ -126,7 +126,7 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
   if ((image->columns % 2) != 0)
@@ -135,54 +135,50 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     return((Image *) NULL);
-  for (i=0; i < image->offset; i++)
-    (void) ReadBlobByte(image);
+  if (DiscardBlobBytes(image,image->offset) == MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   image->depth=8;
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
   /*
     Accumulate UYVY, then unpack into two pixels.
   */
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) (image->columns >> 1); x++)
+    for (x=0; x < (ssize_t) (image->columns >> 1); x++)
     {
       u=(unsigned char) ReadBlobByte(image);
       y1=(unsigned char) ReadBlobByte(image);
       v=(unsigned char) ReadBlobByte(image);
       y2=(unsigned char) ReadBlobByte(image);
-      q->red=ScaleCharToQuantum(y1);
-      q->green=ScaleCharToQuantum(u);
-      q->blue=ScaleCharToQuantum(v);
+      SetPixelRed(q,ScaleCharToQuantum(y1));
+      SetPixelGreen(q,ScaleCharToQuantum(u));
+      SetPixelBlue(q,ScaleCharToQuantum(v));
       q++;
-      q->red=ScaleCharToQuantum(y2);
-      q->green=ScaleCharToQuantum(u);
-      q->blue=ScaleCharToQuantum(v);
+      SetPixelRed(q,ScaleCharToQuantum(y2));
+      SetPixelGreen(q,ScaleCharToQuantum(u));
+      SetPixelBlue(q,ScaleCharToQuantum(v));
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+      break;
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
   }
   image->colorspace=YCbCrColorspace;
-  (void) SetImageColorspace(image,RGBColorspace);
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -206,32 +202,33 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterUYVYImage method is:
 %
-%      RegisterUYVYImage(void)
+%      size_t RegisterUYVYImage(void)
 %
 */
-ModuleExport void RegisterUYVYImage(void)
+ModuleExport size_t RegisterUYVYImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("PAL");
-  entry->decoder=(DecoderHandler *) ReadUYVYImage;
-  entry->encoder=(EncoderHandler *) WriteUYVYImage;
+  entry->decoder=(DecodeImageHandler *) ReadUYVYImage;
+  entry->encoder=(EncodeImageHandler *) WriteUYVYImage;
   entry->adjoin=MagickFalse;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("16bit/pixel interleaved YUV");
-  entry->module=AcquireString("UYVY");
+  entry->description=ConstantString("16bit/pixel interleaved YUV");
+  entry->module=ConstantString("UYVY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("UYVY");
-  entry->decoder=(DecoderHandler *) ReadUYVYImage;
-  entry->encoder=(EncoderHandler *) WriteUYVYImage;
+  entry->decoder=(DecodeImageHandler *) ReadUYVYImage;
+  entry->encoder=(EncodeImageHandler *) WriteUYVYImage;
   entry->adjoin=MagickFalse;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("16bit/pixel interleaved YUV");
-  entry->module=AcquireString("UYVY");
+  entry->description=ConstantString("16bit/pixel interleaved YUV");
+  entry->module=ConstantString("UYVY");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -281,7 +278,7 @@ ModuleExport void UnregisterUYVYImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %      Implicit assumption: number of columns is even.
@@ -296,9 +293,6 @@ static MagickBooleanType WriteUYVYImage(const ImageInfo *image_info,
   Image
     *uyvy_image;
 
-  long
-    y;
-
   MagickBooleanType
     full,
     status;
@@ -306,9 +300,12 @@ static MagickBooleanType WriteUYVYImage(const ImageInfo *image_info,
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     x;
 
+  ssize_t
+    y;
+
   /*
     Open output image file.
   */
@@ -329,41 +326,38 @@ static MagickBooleanType WriteUYVYImage(const ImageInfo *image_info,
   uyvy_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (uyvy_image == (Image *) NULL)
     ThrowWriterException(ResourceLimitError,image->exception.reason);
-  (void) SetImageColorspace(uyvy_image,YCbCrColorspace);
+  (void) TransformImageColorspace(uyvy_image,YCbCrColorspace);
   full=MagickFalse;
   (void) ResetMagickMemory(&pixel,0,sizeof(MagickPixelPacket));
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=AcquireImagePixels(uyvy_image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(uyvy_image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       if (full != MagickFalse)
         {
-          pixel.green=(pixel.green+p->green)/2;
-          pixel.blue=(pixel.blue+p->blue)/2;
-          (void) WriteBlobByte(image,ScaleQuantumToChar(pixel.green));
-          (void) WriteBlobByte(image,ScaleQuantumToChar(pixel.red));
-          (void) WriteBlobByte(image,ScaleQuantumToChar(pixel.blue));
-          (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
+          pixel.green=(pixel.green+GetPixelGreen(p))/2;
+          pixel.blue=(pixel.blue+GetPixelBlue(p))/2;
+          (void) WriteBlobByte(image,ScaleQuantumToChar((Quantum) pixel.green));
+          (void) WriteBlobByte(image,ScaleQuantumToChar((Quantum) pixel.red));
+          (void) WriteBlobByte(image,ScaleQuantumToChar((Quantum) pixel.blue));
+          (void) WriteBlobByte(image,ScaleQuantumToChar(
+             GetPixelRed(p)));
         }
-      pixel.red=(double) p->red;
-      pixel.green=(double) p->green;
-      pixel.blue=(double) p->blue;
-      full=(MagickBooleanType) (full == MagickFalse);
+      pixel.red=(double) GetPixelRed(p);
+      pixel.green=(double) GetPixelGreen(p);
+      pixel.blue=(double) GetPixelBlue(p);
+      full=full == MagickFalse ? MagickTrue : MagickFalse;
       p++;
     }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SaveImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
+      break;
   }
   uyvy_image=DestroyImage(uyvy_image);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
