@@ -10,14 +10,14 @@
 %                       V    IIIII   CCCC  A   A  R  R                        %
 %                                                                             %
 %                                                                             %
-%                    Read/Write VICAR Rasterfile Format.                      %
+%                    Read/Write VICAR Rasterfile Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,7 +42,10 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/colormap.h"
 #include "magick/colorspace.h"
+#include "magick/colorspace-private.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -51,10 +54,14 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/module.h"
 #include "magick/monitor.h"
-#include "magick/pixel.h"
+#include "magick/monitor-private.h"
+#include "magick/quantum-private.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
 
 /*
   Forward declarations.
@@ -78,24 +85,26 @@ static MagickBooleanType
 %
 %  The format of the IsVICAR method is:
 %
-%      MagickBooleanType IsVICAR(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsVICAR(const unsigned char *magick,
+%        const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
-static MagickBooleanType IsVICAR(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsVICAR(const unsigned char *magick,
+  const size_t length)
 {
-  if (length < 7)
+  if (length < 14)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick,"LBLSIZE",7) == 0)
+  if (LocaleNCompare((const char *) magick,"LBLSIZE",7) == 0)
     return(MagickTrue);
-  if (LocaleNCompare((char *) magick,"NJPL1I",6) == 0)
+  if (LocaleNCompare((const char *) magick,"NJPL1I",6) == 0)
+    return(MagickTrue);
+  if (LocaleNCompare((const char *) magick,"PDS_VERSION_ID",14) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -126,7 +135,7 @@ static MagickBooleanType IsVICAR(const unsigned char *magick,const size_t length
 %      reading.  A null image is returned if there is a memory shortage or if
 %      the image cannot be read.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -145,24 +154,28 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   int
     c;
 
-  long
-    y;
-
   MagickBooleanType
     status,
     value_expected;
 
+  QuantumInfo
+    *quantum_info;
+
+  QuantumType
+    quantum_type;
+
   register PixelPacket
     *q;
 
-  ssize_t
-    count;
-
-  ssize_t
+  size_t
     length;
 
+  ssize_t
+    count,
+    y;
+
   unsigned char
-    *scanline;
+    *pixels;
 
   /*
     Open image file.
@@ -174,7 +187,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -212,7 +225,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
         p=keyword;
         do
         {
-          if ((size_t) (p-keyword) < (MaxTextExtent/2))
+          if ((size_t) (p-keyword) < (MaxTextExtent-1))
             *p++=c;
           c=ReadBlobByte(image);
           count++;
@@ -231,7 +244,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
         p=value;
         while (isalnum(c))
         {
-          if ((size_t) (p-value) < (MaxTextExtent/2))
+          if ((size_t) (p-value) < (MaxTextExtent-1))
             *p++=c;
           c=ReadBlobByte(image);
           count++;
@@ -241,17 +254,17 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
           Assign a value to the specified keyword.
         */
         if (LocaleCompare(keyword,"Label_RECORDS") == 0)
-          length=atol(value);
+          length=(ssize_t) StringToLong(value);
         if (LocaleCompare(keyword,"LBLSIZE") == 0)
-          length=atol(value);
+          length=(ssize_t) StringToLong(value);
         if (LocaleCompare(keyword,"RECORD_BYTES") == 0)
-          image->columns= atol(value);
+          image->columns=StringToUnsignedLong(value);
         if (LocaleCompare(keyword,"NS") == 0)
-          image->columns= atol(value);
+          image->columns=StringToUnsignedLong(value);
         if (LocaleCompare(keyword,"LINES") == 0)
-          image->rows= atol(value);
+          image->rows=StringToUnsignedLong(value);
         if (LocaleCompare(keyword,"NL") == 0)
-          image->rows= atol(value);
+          image->rows=StringToUnsignedLong(value);
       }
     while (isspace((int) ((unsigned char) c)) != 0)
     {
@@ -259,7 +272,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
       count++;
     }
   }
-  while (count < (long) length)
+  while (count < (ssize_t) length)
   {
     c=ReadBlobByte(image);
     count++;
@@ -267,42 +280,43 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
   image->depth=8;
-  if (AllocateImageColormap(image,256) == MagickFalse)
+  if (AcquireImageColormap(image,256) == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
   /*
     Read VICAR pixels.
   */
-  scanline=(unsigned char *) AcquireMagickMemory(image->columns);
-  if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(CorruptImageError,"UnableToReadImageData");
-  for (y=0; y < (long) image->rows; y++)
+  quantum_type=IndexQuantum;
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  pixels=GetQuantumPixels(quantum_info);
+  length=GetQuantumExtent(image,quantum_info,IndexQuantum);
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    count=ReadBlob(image,image->columns,scanline);
-    (void) ExportQuantumPixels(image,GrayQuantum,0,scanline);
-    if (SyncImagePixels(image) == MagickFalse)
+    count=ReadBlob(image,length,pixels);
+    (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
+      quantum_type,pixels,exception);
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+      break;
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
   }
-  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+  SetQuantumImageType(image,quantum_type);
+  quantum_info=DestroyQuantumInfo(quantum_info);
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -326,22 +340,23 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterVICARImage method is:
 %
-%      RegisterVICARImage(void)
+%      size_t RegisterVICARImage(void)
 %
 */
-ModuleExport void RegisterVICARImage(void)
+ModuleExport size_t RegisterVICARImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("VICAR");
-  entry->decoder=(DecoderHandler *) ReadVICARImage;
-  entry->encoder=(EncoderHandler *) WriteVICARImage;
-  entry->magick=(MagickHandler *) IsVICAR;
+  entry->decoder=(DecodeImageHandler *) ReadVICARImage;
+  entry->encoder=(EncodeImageHandler *) WriteVICARImage;
+  entry->magick=(IsImageFormatHandler *) IsVICAR;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("VICAR rasterfile format");
-  entry->module=AcquireString("VICAR");
+  entry->description=ConstantString("VICAR rasterfile format");
+  entry->module=ConstantString("VICAR");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -385,8 +400,7 @@ ModuleExport void UnregisterVICARImage(void)
 %  stacked together to form image cubes.  This method only writes a single
 %  grayscale plane.
 %
-%  WriteVICARImage was written contributed by
-%  gorelick@esther.la.asu.edu.
+%  WriteVICARImage was written contributed by gorelick@esther.la.asu.edu.
 %
 %  The format of the WriteVICARImage method is:
 %
@@ -395,11 +409,10 @@ ModuleExport void UnregisterVICARImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   Image *image)
@@ -413,11 +426,20 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  QuantumInfo
+    *quantum_info;
+
   register const PixelPacket
     *p;
 
+  size_t
+    length;
+
+  ssize_t
+    count;
+
   unsigned char
-    *scanline;
+    *pixels;
 
   /*
     Open output image file.
@@ -431,45 +453,42 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (IsRGBColorspace(image->colorspace) == MagickFalse)
+    (void) TransformImageColorspace(image,RGBColorspace);
   /*
     Write header.
   */
   (void) ResetMagickMemory(header,' ',MaxTextExtent);
-  (void) FormatMagickString(header,MaxTextExtent,
-    "LBLSIZE=%lu FORMAT='BYTE' TYPE='IMAGE' BUFSIZE=20000 DIM=2 EOL=0 "
-    "RECSIZE=%lu ORG='BSQ' NL=%lu NS=%lu NB=1 N1=0 N2=0 N3=0 N4=0 NBB=0 "
-    "NLB=0 TASK='ImageMagick'",(unsigned long) MaxTextExtent,image->columns,
-    image->rows,image->columns);
+  (void) FormatLocaleString(header,MaxTextExtent,
+    "LBLSIZE=%.20g FORMAT='BYTE' TYPE='IMAGE' BUFSIZE=20000 DIM=2 EOL=0 "
+    "RECSIZE=%.20g ORG='BSQ' NL=%.20g NS=%.20g NB=1 N1=0 N2=0 N3=0 N4=0 NBB=0 "
+    "NLB=0 TASK='ImageMagick'",(double) MaxTextExtent,(double) image->columns,
+    (double) image->rows,(double) image->columns);
   (void) WriteBlob(image,MaxTextExtent,(unsigned char *) header);
   /*
-    Allocate memory for scanline.
-  */
-  scanline=(unsigned char *) AcquireMagickMemory(image->columns);
-  if (scanline == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  /*
-    Write VICAR scanline.
+    Write VICAR pixels.
   */
   image->depth=8;
-  for (y=0; y < (long) image->rows; y++)
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  pixels=GetQuantumPixels(quantum_info);
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ImportQuantumPixels(image,GrayQuantum,0,scanline);
-    (void) WriteBlob(image,image->columns,scanline);
-    if (image->previous == (Image *) NULL)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+      GrayQuantum,pixels,&image->exception);
+    count=WriteBlob(image,length,pixels);
+    if (count != (ssize_t) length)
+      break;
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
+      break;
   }
-  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
-  CloseBlob(image);
+  quantum_info=DestroyQuantumInfo(quantum_info);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
