@@ -10,14 +10,14 @@
 %                            X   X  BBBB   M   M                              %
 %                                                                             %
 %                                                                             %
-%                  Read/Write X Windows System Bitmap Format.                 %
+%                  Read/Write X Windows System Bitmap Format                  %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,8 +42,11 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color-private.h"
+#include "magick/colormap.h"
 #include "magick/colorspace.h"
+#include "magick/colorspace-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
@@ -52,8 +55,11 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/utility.h"
 
 /*
@@ -82,12 +88,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsXBM(const unsigned char *magick,const size_t length)
 {
@@ -119,11 +123,10 @@ static MagickBooleanType IsXBM(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 
 static int XBMInteger(Image *image,short int *hex_digits)
@@ -146,7 +149,7 @@ static int XBMInteger(Image *image,short int *hex_digits)
     c&=0xff;
     if (isxdigit(c) != MagickFalse)
       {
-        value=(int) ((unsigned long) value << 4)+hex_digits[c];
+        value=(int) ((size_t) value << 4)+hex_digits[c];
         flag++;
         continue;
       }
@@ -165,16 +168,13 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
   register IndexPacket
     *indexes;
 
-  register long
+  register ssize_t
     i,
     x;
 
@@ -187,16 +187,24 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   short int
     hex_digits[256];
 
+  size_t
+    bit,
+    byte,
+    bytes_per_line,
+    length,
+    padding,
+    value,
+    version;
+
+  ssize_t
+    y;
+
   unsigned char
     *data;
 
   unsigned long
-    bit,
-    byte,
-    bytes_per_line,
-    padding,
-    value,
-    version;
+    height,
+    width;
 
   /*
     Open image file.
@@ -208,7 +216,7 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -218,16 +226,20 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read X bitmap header.
   */
+  width=0;
+  height=0;
   while (ReadBlobString(image,buffer) != (char *) NULL)
-    if (sscanf(buffer,"#define %s %lu",name,&image->columns) == 2)
+    if (sscanf(buffer,"#define %s %lu",name,&width) == 2)
       if ((strlen(name) >= 6) &&
           (LocaleCompare(name+strlen(name)-6,"_width") == 0))
-          break;
+        break;
   while (ReadBlobString(image,buffer) != (char *) NULL)
-    if (sscanf(buffer,"#define %s %lu",name,&image->rows) == 2)
+    if (sscanf(buffer,"#define %s %lu",name,&height) == 2)
       if ((strlen(name) >= 7) &&
           (LocaleCompare(name+strlen(name)-7,"_height") == 0))
-          break;
+        break;
+  image->columns=width;
+  image->rows=height;
   image->depth=8;
   image->storage_class=PseudoClass;
   image->colors=2;
@@ -261,29 +273,20 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Initialize image structure.
   */
-  if (AllocateImageColormap(image,image->colors) == MagickFalse)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  padding=0;
-  if (((image->columns % 16) != 0) &&
-      ((image->columns % 16) < 9) && (version == 10))
-    padding=1;
-  bytes_per_line=(image->columns+7)/8+padding;
-  data=(unsigned char *)
-    AcquireMagickMemory((size_t) bytes_per_line*image->rows);
-  if (data == (unsigned char *) NULL)
+  if (AcquireImageColormap(image,image->colors) == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Initialize colormap.
   */
-  image->colormap[0].red=QuantumRange;
-  image->colormap[0].green=QuantumRange;
-  image->colormap[0].blue=QuantumRange;
-  image->colormap[1].red=0;
-  image->colormap[1].green=0;
-  image->colormap[1].blue=0;
+  image->colormap[0].red=(Quantum) QuantumRange;
+  image->colormap[0].green=(Quantum) QuantumRange;
+  image->colormap[0].blue=(Quantum) QuantumRange;
+  image->colormap[1].red=(Quantum) 0;
+  image->colormap[1].green=(Quantum) 0;
+  image->colormap[1].blue=(Quantum) 0;
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
   /*
@@ -320,58 +323,63 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read hex image data.
   */
+  padding=0;
+  if (((image->columns % 16) != 0) &&
+      ((image->columns % 16) < 9) && (version == 10))
+    padding=1;
+  bytes_per_line=(image->columns+7)/8+padding;
+  length=(size_t) image->rows;
+  data=(unsigned char *) AcquireQuantumMemory(length,bytes_per_line*
+    sizeof(*data));
+  if (data == (unsigned char *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   p=data;
   if (version == 10)
-    for (i=0; i < (long) (bytes_per_line*image->rows); (i+=2))
+    for (i=0; i < (ssize_t) (bytes_per_line*image->rows); (i+=2))
     {
-      value=(unsigned long) XBMInteger(image,hex_digits);
+      value=(size_t) XBMInteger(image,hex_digits);
       *p++=(unsigned char) value;
       if ((padding == 0) || (((i+2) % bytes_per_line) != 0))
         *p++=(unsigned char) (value >> 8);
     }
   else
-    for (i=0; i < (long) (bytes_per_line*image->rows); i++)
+    for (i=0; i < (ssize_t) (bytes_per_line*image->rows); i++)
     {
-      value=(unsigned long) XBMInteger(image,hex_digits);
+      value=(size_t) XBMInteger(image,hex_digits);
       *p++=(unsigned char) value;
     }
   /*
     Convert X bitmap image to pixel packets.
   */
   p=data;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetAuthenticIndexQueue(image);
     bit=0;
     byte=0;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       if (bit == 0)
-        byte=(unsigned long) (*p++);
-      indexes[x]=(IndexPacket) ((byte & 0x01) != 0 ? 0x01 : 0x00);
+        byte=(size_t) (*p++);
+      SetPixelIndex(indexes+x,(byte & 0x01) != 0 ? 0x01 : 0x00);
       bit++;
       byte>>=1;
       if (bit == 8)
         bit=0;
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+      break;
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(LoadImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
   }
   data=(unsigned char *) RelinquishMagickMemory(data);
-  SyncImage(image);
-  CloseBlob(image);
+  (void) SyncImage(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -395,23 +403,24 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterXBMImage method is:
 %
-%      RegisterXBMImage(void)
+%      size_t RegisterXBMImage(void)
 %
 */
-ModuleExport void RegisterXBMImage(void)
+ModuleExport size_t RegisterXBMImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("XBM");
-  entry->decoder=(DecoderHandler *) ReadXBMImage;
-  entry->encoder=(EncoderHandler *) WriteXBMImage;
-  entry->magick=(MagickHandler *) IsXBM;
+  entry->decoder=(DecodeImageHandler *) ReadXBMImage;
+  entry->encoder=(EncodeImageHandler *) WriteXBMImage;
+  entry->magick=(IsImageFormatHandler *) IsXBM;
   entry->adjoin=MagickFalse;
-  entry->description=
-    AcquireString("X Windows system bitmap (black and white)");
-  entry->module=AcquireString("XBM");
+  entry->description=ConstantString(
+    "X Windows system bitmap (black and white)");
+  entry->module=ConstantString("XBM");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -457,7 +466,7 @@ ModuleExport void UnregisterXBMImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -469,31 +478,23 @@ static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
     basename[MaxTextExtent],
     buffer[MaxTextExtent];
 
-  IndexPacket
-    polarity;
-
-  long
-    y;
-
   MagickBooleanType
     status;
 
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     x;
 
-  register IndexPacket
-    *indexes;
-
-  ssize_t
-    count;
-
-  unsigned long
+  size_t
     bit,
     byte;
 
+  ssize_t
+    count,
+    y;
+
   /*
     Open output image file.
   */
@@ -506,48 +507,43 @@ static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (IsRGBColorspace(image->colorspace) == MagickFalse)
+    (void) TransformImageColorspace(image,RGBColorspace);
   /*
     Write X bitmap header.
   */
   GetPathComponent(image->filename,BasePath,basename);
-  (void) FormatMagickString(buffer,MaxTextExtent,"#define %s_width %lu\n",
-    basename,image->columns);
+  (void) FormatLocaleString(buffer,MaxTextExtent,"#define %s_width %.20g\n",
+    basename,(double) image->columns);
   (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
-  (void) FormatMagickString(buffer,MaxTextExtent,"#define %s_height %lu\n",
-    basename,image->rows);
+  (void) FormatLocaleString(buffer,MaxTextExtent,"#define %s_height %.20g\n",
+    basename,(double) image->rows);
   (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
-  (void) FormatMagickString(buffer,MaxTextExtent,
+  (void) FormatLocaleString(buffer,MaxTextExtent,
     "static char %s_bits[] = {\n",basename);
   (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
-  (void) strcpy(buffer," ");
+  (void) CopyMagickString(buffer," ",MaxTextExtent);
   (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
   /*
     Convert MIFF to X bitmap pixels.
   */
   (void) SetImageType(image,BilevelType);
-  polarity=(IndexPacket)
-    (PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2));
-  if (image->colors == 2)
-    polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
-      PixelIntensityToQuantum(&image->colormap[1]));
   bit=0;
   byte=0;
   count=0;
   x=0;
   y=0;
-  (void) strcpy(buffer," ");
+  (void) CopyMagickString(buffer," ",MaxTextExtent);
   (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       byte>>=1;
-      if (indexes[x] != polarity)
+      if (PixelIntensity(p) < ((MagickRealType) QuantumRange/2.0))
         byte|=0x80;
       bit++;
       if (bit == 8)
@@ -555,13 +551,13 @@ static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
           /*
             Write a bitmap byte to the image file.
           */
-          (void) FormatMagickString(buffer,MaxTextExtent,"0x%02X, ",
+          (void) FormatLocaleString(buffer,MaxTextExtent,"0x%02X, ",
             (unsigned int) (byte & 0xff));
           (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
           count++;
           if (count == 12)
             {
-              (void) strcpy(buffer,"\n  ");
+              (void) CopyMagickString(buffer,"\n  ",MaxTextExtent);
               (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
               count=0;
             };
@@ -576,31 +572,26 @@ static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
           Write a bitmap byte to the image file.
         */
         byte>>=(8-bit);
-        (void) FormatMagickString(buffer,MaxTextExtent,"0x%02X, ",
+        (void) FormatLocaleString(buffer,MaxTextExtent,"0x%02X, ",
           (unsigned int) (byte & 0xff));
         (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
         count++;
         if (count == 12)
           {
-            (void) strcpy(buffer,"\n  ");
+            (void) CopyMagickString(buffer,"\n  ",MaxTextExtent);
             (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
             count=0;
           };
         bit=0;
         byte=0;
       };
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
+      break;
   }
-  (void) strcpy(buffer,"};\n");
+  (void) CopyMagickString(buffer,"};\n",MaxTextExtent);
   (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
