@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -52,21 +52,12 @@ extern "C" {
 #define LiberateMagickResource(resource)  RelinquishMagickResource(resource)
 #define LiberateSemaphore(semaphore)  RelinquishSemaphore(semaphore)
 #define QuantumDepth  MAGICKCORE_QUANTUM_DEPTH
+#define MaxRGB  QuantumRange  /* deprecated */
 #define RunlengthEncodedCompression  RLECompression
 #define Upscale(value)  ScaleCharToQuantum(value)
 #define XDownscale(value)  ScaleShortToQuantum(value)
 #define XUpscale(quantum)  ScaleQuantumToShort(quantum)
 
-typedef struct _DoublePixelPacket
-{
-  double
-    red,
-    green,
-    blue,
-    opacity,
-    index;
-} DoublePixelPacket;
-
 typedef enum
 {
   UndefinedMagickLayerMethod
@@ -95,6 +86,10 @@ typedef struct _ImageAttribute
     *next;  /* deprecated */
 } ImageAttribute;
 
+extern MagickExport CacheView
+  *CloseCacheView(CacheView *) magick_attribute((deprecated)),
+  *OpenCacheView(const Image *) magick_attribute((deprecated));
+
 extern MagickExport char
   *AllocateString(const char *) magick_attribute((deprecated)),
   *InterpretImageAttributes(const ImageInfo *,Image *,const char *)
@@ -114,71 +109,80 @@ extern MagickExport const IndexPacket
 
 extern MagickExport const PixelPacket
   *AcquirePixels(const Image *) magick_attribute((deprecated)),
-  *AcquireCacheViewPixels(const CacheView *,const long,const long,
-    const unsigned long,const unsigned long,ExceptionInfo *)
+  *AcquireCacheViewPixels(const CacheView *,const ssize_t,const ssize_t,
+    const size_t,const size_t,ExceptionInfo *)
     magick_attribute((deprecated)),
-  *AcquireImagePixels(const Image *,const long,const long,const unsigned long,
-    const unsigned long,ExceptionInfo *) magick_attribute((deprecated));
+  *AcquireImagePixels(const Image *,const ssize_t,const ssize_t,const size_t,
+    const size_t,ExceptionInfo *) magick_attribute((deprecated));
+
+extern MagickExport FILE
+  *OpenMagickStream(const char *,const char *);
 
 extern MagickExport Image
   *AllocateImage(const ImageInfo *) magick_attribute((deprecated)),
+  *AverageImages(const Image *,ExceptionInfo *),
   *ExtractSubimageFromImage(Image *,const Image *,ExceptionInfo *)
     magick_attribute((deprecated)),
-  *GetImageFromMagickRegistry(const char *,long *id,ExceptionInfo *)
+  *GetImageFromMagickRegistry(const char *,ssize_t *id,ExceptionInfo *)
     magick_attribute((deprecated)),
-  *GetImageList(const Image *,const long,ExceptionInfo *)
+  *GetImageList(const Image *,const ssize_t,ExceptionInfo *)
     magick_attribute((deprecated)),
   *GetNextImage(const Image *) magick_attribute((deprecated)),
   *GetPreviousImage(const Image *) magick_attribute((deprecated)),
   *FlattenImages(Image *,ExceptionInfo *) magick_attribute((deprecated)),
+  *MaximumImages(const Image *,ExceptionInfo *),
+  *MedianFilterImage(const Image *,const double,ExceptionInfo *),
+  *ModeImage(const Image *,const double,ExceptionInfo *),
+  *MinimumImages(const Image *,ExceptionInfo *),
   *MosaicImages(Image *,ExceptionInfo *) magick_attribute((deprecated)),
   *PopImageList(Image **) magick_attribute((deprecated)),
+  *RecolorImage(const Image *,const size_t,const double *,ExceptionInfo *)
+    magick_attribute((deprecated)),
+  *ReduceNoiseImage(const Image *,const double,ExceptionInfo *),
   *ShiftImageList(Image **) magick_attribute((deprecated)),
-  *SpliceImageList(Image *,const long,const unsigned long,const Image *,
-    ExceptionInfo *) magick_attribute((deprecated));
+  *SpliceImageList(Image *,const ssize_t,const size_t,const Image *,
+    ExceptionInfo *) magick_attribute((deprecated)),
+  *ZoomImage(const Image *,const size_t,const size_t,ExceptionInfo *)
+    magick_attribute((deprecated));
 
 extern MagickExport IndexPacket
   *GetCacheViewIndexes(CacheView *) magick_attribute((deprecated)),
   *GetIndexes(const Image *) magick_attribute((deprecated)),
-  ValidateColormapIndex(Image *,const unsigned long)
-    magick_attribute((deprecated));
+  ValidateColormapIndex(Image *,const size_t) magick_attribute((deprecated));
 
 extern MagickExport int
   GetImageGeometry(Image *,const char *,const unsigned int,RectangleInfo *)
     magick_attribute((deprecated)),
-  ParseImageGeometry(const char *,long *,long *,unsigned long *,
-    unsigned long *) magick_attribute((deprecated));
-
-extern MagickExport long
-  GetImageListIndex(const Image *) magick_attribute((deprecated)),
-  SetMagickRegistry(const RegistryType,const void *,const size_t,
-    ExceptionInfo *) magick_attribute((deprecated));
+  ParseImageGeometry(const char *,ssize_t *,ssize_t *,size_t *,size_t *)
+    magick_attribute((deprecated));
 
 extern MagickExport MagickBooleanType
-  AcquireOneCacheViewPixel(const CacheView *,const long,const long,
+  AcquireOneCacheViewPixel(const CacheView *,const ssize_t,const ssize_t,
     PixelPacket *,ExceptionInfo *) magick_attribute((deprecated)),
   AcquireOneCacheViewVirtualPixel(const CacheView *,const VirtualPixelMethod,
-    const long,const long,PixelPacket *,ExceptionInfo *)
+    const ssize_t,const ssize_t,PixelPacket *,ExceptionInfo *)
     magick_attribute((deprecated)),
   AffinityImage(const QuantizeInfo *,Image *,const Image *)
     magick_attribute((deprecated)),
   AffinityImages(const QuantizeInfo *,Image *,const Image *)
     magick_attribute((deprecated)),
-  AllocateImageColormap(Image *,const unsigned long)
+  AllocateImageColormap(Image *,const size_t)
     magick_attribute((deprecated)),
   ClipPathImage(Image *,const char *,const MagickBooleanType)
     magick_attribute((deprecated)),
   CloneImageAttributes(Image *,const Image *) magick_attribute((deprecated)),
-  ColorFloodfillImage(Image *,const DrawInfo *,const PixelPacket,const long,
-    const long,const PaintMethod) magick_attribute((deprecated)),
+  ColorFloodfillImage(Image *,const DrawInfo *,const PixelPacket,const ssize_t,
+    const ssize_t,const PaintMethod) magick_attribute((deprecated)),
   DeleteImageAttribute(Image *,const char *) magick_attribute((deprecated)),
-  DeleteMagickRegistry(const long) magick_attribute((deprecated)),
+  DeleteMagickRegistry(const ssize_t) magick_attribute((deprecated)),
   DescribeImage(Image *,FILE *,const MagickBooleanType)
     magick_attribute((deprecated)),
   FormatImageAttribute(Image *,const char *,const char *,...)
     magick_attribute((format (printf,3,4))) magick_attribute((deprecated)),
   FormatImageAttributeList(Image *,const char *,const char *,va_list)
     magick_attribute((format (printf,3,0))) magick_attribute((deprecated)),
+  FormatImagePropertyList(Image *,const char *,const char *,va_list)
+    magick_attribute((format (printf,3,0))),
   FuzzyColorCompare(const Image *,const PixelPacket *,const PixelPacket *)
     magick_attribute((deprecated)),
   FuzzyOpacityCompare(const Image *,const PixelPacket *,const PixelPacket *)
@@ -192,12 +196,12 @@ extern MagickExport MagickBooleanType
     magick_attribute((deprecated)),
   MapImages(Image *,const Image *,const MagickBooleanType)
     magick_attribute((deprecated)),
-  MatteFloodfillImage(Image *,const PixelPacket,const Quantum,const long,
-    const long,const PaintMethod) magick_attribute((deprecated)),
+  MatteFloodfillImage(Image *,const PixelPacket,const Quantum,const ssize_t,
+    const ssize_t,const PaintMethod) magick_attribute((deprecated)),
   OpaqueImage(Image *,const PixelPacket,const PixelPacket)
     magick_attribute((deprecated)),
   PaintFloodfillImage(Image *,const ChannelType,const MagickPixelPacket *,
-    const long,const long,const DrawInfo *,const PaintMethod)
+    const ssize_t,const ssize_t,const DrawInfo *,const PaintMethod)
     magick_attribute((deprecated)),
   PaintOpaqueImage(Image *,const MagickPixelPacket *,const MagickPixelPacket *)
     magick_attribute((deprecated)),
@@ -215,8 +219,8 @@ extern MagickExport MagickBooleanType
     magick_attribute((deprecated));
 
 extern MagickExport MagickPixelPacket
-  AcquireOneMagickPixel(const Image *,const long,const long,ExceptionInfo *)
-    magick_attribute((deprecated));
+  AcquireOneMagickPixel(const Image *,const ssize_t,const ssize_t,
+    ExceptionInfo *) magick_attribute((deprecated));
 
 extern MagickExport MonitorHandler
   GetMonitorHandler(void) magick_attribute((deprecated)),
@@ -234,39 +238,50 @@ extern MagickExport MagickStatusType
     magick_attribute((deprecated));
 
 extern MagickExport PixelPacket
-  AcquireOnePixel(const Image *,const long,const long,ExceptionInfo *)
+  AcquireOnePixel(const Image *,const ssize_t,const ssize_t,ExceptionInfo *)
+    magick_attribute((deprecated)),
+  AcquireOneVirtualPixel(const Image *,const VirtualPixelMethod,const ssize_t,
+    const ssize_t,ExceptionInfo *) magick_attribute((deprecated)),
+  *GetCacheView(CacheView *,const ssize_t,const ssize_t,const size_t,
+    const size_t) magick_attribute((deprecated)),
+  *GetCacheViewPixels(CacheView *,const ssize_t,const ssize_t,const size_t,
+    const size_t) magick_attribute((deprecated)),
+  *GetImagePixels(Image *,const ssize_t,const ssize_t,const size_t,
+    const size_t) magick_attribute((deprecated)),
+  GetOnePixel(Image *,const ssize_t,const ssize_t)
     magick_attribute((deprecated)),
-  AcquireOneVirtualPixel(const Image *,const VirtualPixelMethod,const long,
-    const long,ExceptionInfo *) magick_attribute((deprecated)),
-  *GetCacheView(CacheView *,const long,const long,const unsigned long,
-    const unsigned long) magick_attribute((deprecated)),
-  *GetCacheViewPixels(CacheView *,const long,const long,const unsigned long,
-    const unsigned long) magick_attribute((deprecated)),
-  *GetImagePixels(Image *,const long,const long,const unsigned long,
-    const unsigned long) magick_attribute((deprecated)),
-  GetOnePixel(Image *,const long,const long) magick_attribute((deprecated)),
   *GetPixels(const Image *) magick_attribute((deprecated)),
-  *SetCacheViewPixels(CacheView *,const long,const long,const unsigned long,
-    const unsigned long) magick_attribute((deprecated)),
-  *SetImagePixels(Image *,const long,const long,const unsigned long,
-    const unsigned long) magick_attribute((deprecated));
+  *SetCacheViewPixels(CacheView *,const ssize_t,const ssize_t,const size_t,
+    const size_t) magick_attribute((deprecated)),
+  *SetImagePixels(Image *,const ssize_t,const ssize_t,const size_t,
+    const size_t) magick_attribute((deprecated));
 
 extern MagickExport size_t
+  GetImageListSize(const Image *) magick_attribute((deprecated)),
   PopImagePixels(Image *,const QuantumType,unsigned char *)
     magick_attribute((deprecated)),
   PushImagePixels(Image *,const QuantumType,const unsigned char *)
     magick_attribute((deprecated));
 
+extern MagickExport ssize_t
+  FormatMagickString(char *,const size_t,const char *,...)
+    magick_attribute((format (printf,3,4))) magick_attribute((deprecated)),
+  FormatMagickStringList(char *,const size_t,const char *,va_list)
+    magick_attribute((format (printf,3,0))),
+  GetImageListIndex(const Image *) magick_attribute((deprecated)),
+  SetMagickRegistry(const RegistryType,const void *,const size_t,
+    ExceptionInfo *) magick_attribute((deprecated));
+
 extern MagickExport unsigned int
   ChannelImage(Image *,const ChannelType) magick_attribute((deprecated)),
   ChannelThresholdImage(Image *,const char *) magick_attribute((deprecated)),
-  DispatchImage(const Image *,const long,const long,const unsigned long,
-    const unsigned long,const char *,const StorageType,void *,ExceptionInfo *)
+  DispatchImage(const Image *,const ssize_t,const ssize_t,const size_t,
+    const size_t,const char *,const StorageType,void *,ExceptionInfo *)
     magick_attribute((deprecated)),
   FuzzyColorMatch(const PixelPacket *,const PixelPacket *,const double)
     magick_attribute((deprecated)),
   GetNumberScenes(const Image *) magick_attribute((deprecated)),
-  GetMagickGeometry(const char *,long *,long *,unsigned long *,unsigned long *)
+  GetMagickGeometry(const char *,ssize_t *,ssize_t *,size_t *,size_t *)
     magick_attribute((deprecated)),
   IsSubimage(const char *,const unsigned int) magick_attribute((deprecated)),
   PushImageList(Image **,const Image *,ExceptionInfo *)
@@ -274,7 +289,7 @@ extern MagickExport unsigned int
   QuantizationError(Image *) magick_attribute((deprecated)),
   RandomChannelThresholdImage(Image *,const char *,const char *,
     ExceptionInfo *) magick_attribute((deprecated)),
-  SetImageList(Image **,const Image *,const long,ExceptionInfo *)
+  SetImageList(Image **,const Image *,const ssize_t,ExceptionInfo *)
     magick_attribute((deprecated)),
   TransformColorspace(Image *,const ColorspaceType)
     magick_attribute((deprecated)),
@@ -283,24 +298,18 @@ extern MagickExport unsigned int
   UnshiftImageList(Image **,const Image *,ExceptionInfo *)
     magick_attribute((deprecated));
 
-extern MagickExport unsigned long
-  GetImageListSize(const Image *) magick_attribute((deprecated));
-
-extern MagickExport CacheView
-  *CloseCacheView(CacheView *) magick_attribute((deprecated)),
-  *OpenCacheView(const Image *) magick_attribute((deprecated));
-
 extern MagickExport void
   *AcquireMemory(const size_t) magick_attribute((deprecated)),
   AllocateNextImage(const ImageInfo *,Image *) magick_attribute((deprecated)),
   *CloneMemory(void *,const void *,const size_t) magick_attribute((deprecated)),
+  DestroyConstitute(void),
   DestroyImageAttributes(Image *) magick_attribute((deprecated)),
   DestroyImages(Image *) magick_attribute((deprecated)),
   DestroyMagick(void) magick_attribute((deprecated)),
   DestroyMagickRegistry(void) magick_attribute((deprecated)),
   *GetConfigureBlob(const char *,char *,size_t *,ExceptionInfo *)
     magick_attribute((deprecated)),
-  *GetMagickRegistry(const long,RegistryType *,size_t *,ExceptionInfo *)
+  *GetMagickRegistry(const ssize_t,RegistryType *,size_t *,ExceptionInfo *)
     magick_attribute((deprecated)),
   IdentityAffine(AffineMatrix *) magick_attribute((deprecated)),
   LiberateMemory(void **) magick_attribute((deprecated)),
@@ -312,9 +321,10 @@ extern MagickExport void
   HSLTransform(const double,const double,const double,Quantum *,Quantum *,
     Quantum *) magick_attribute((deprecated)),
   InitializeMagick(const char *) magick_attribute((deprecated)),
+  MagickIncarnate(const char *) magick_attribute((deprecated)),
   ReacquireMemory(void **,const size_t) magick_attribute((deprecated)),
   ResetImageAttributeIterator(const Image *) magick_attribute((deprecated)),
-  SetCacheThreshold(const unsigned long) magick_attribute((deprecated)),
+  SetCacheThreshold(const size_t) magick_attribute((deprecated)),
   SetImage(Image *,const Quantum) magick_attribute((deprecated)),
   Strip(char *) magick_attribute((deprecated)),
   TemporaryFilename(char *) magick_attribute((deprecated)),
