@@ -2,6 +2,7 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 %                    M   M   AAA    GGGG  IIIII   CCCC                        %
 %                    MM MM  A   A  G        I    C                            %
 %                    M M M  AAAAA  G GGG    I    C                            %
@@ -16,7 +17,7 @@
 %                                 July 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,6 +50,7 @@
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
 #include "magick/token.h"
 #include "magick/utility.h"
 #include "magick/xml-tree.h"
@@ -83,7 +85,6 @@ typedef struct _MagicMapInfo
 static const MagicMapInfo
   MagicMap[] =
   {
-    { "AVI", 0, MagickString("RIFF") },
     { "8BIMWTEXT", 0, MagickString("8\000B\000I\000M\000#") },
     { "8BIMTEXT", 0, MagickString("8BIM#") },
     { "8BIM", 0, MagickString("8BIM") },
@@ -119,8 +120,9 @@ static const MagicMapInfo
     { "GIF", 0, MagickString("GIF8") },
     { "GPLT", 0, MagickString("#!/usr/local/bin/gnuplot") },
     { "HDF", 1, MagickString("HDF") },
+    { "HDR", 0, MagickString("#?RADIANCE") },
+    { "HDR", 0, MagickString("#?RGBE") },
     { "HPGL", 0, MagickString("IN;") },
-    { "HPGL", 0, MagickString("\033E\033") },
     { "HTML", 1, MagickString("HTML") },
     { "HTML", 1, MagickString("html") },
     { "ILBM", 8, MagickString("ILBM") },
@@ -131,6 +133,7 @@ static const MagicMapInfo
     { "JPEG", 0, MagickString("\377\330\377") },
     { "JPC", 0, MagickString("\377\117") },
     { "JP2", 4, MagickString("\152\120\040\040\015") },
+    { "MAT", 0, MagickString("MATLAB 5.0 MAT-file,") },
     { "MIFF", 0, MagickString("Id=ImageMagick") },
     { "MIFF", 0, MagickString("id=ImageMagick") },
     { "MNG", 0, MagickString("\212MNG\r\n\032\n") },
@@ -145,28 +148,28 @@ static const MagicMapInfo
     { "PCX", 0, MagickString("\012\005") },
     { "PDB", 60, MagickString("vIMGView") },
     { "PDF", 0, MagickString("%PDF-") },
+    { "PES", 0, MagickString("#PES") },
     { "PFA", 0, MagickString("%!PS-AdobeFont-1.0") },
     { "PFB", 6, MagickString("%!PS-AdobeFont-1.0") },
     { "PGX", 0, MagickString("\050\107\020\115\046") },
     { "PICT", 522, MagickString("\000\021\002\377\014\000") },
     { "PNG", 0, MagickString("\211PNG\r\n\032\n") },
-    { "PNM", 0, MagickString("P1") },
-    { "PNM", 0, MagickString("P2") },
-    { "PNM", 0, MagickString("P3") },
-    { "PNM", 0, MagickString("P4") },
-    { "PNM", 0, MagickString("P5") },
-    { "PNM", 0, MagickString("P6") },
-    { "PNM", 0, MagickString("P7") },
-    { "PNM", 0, MagickString("PF") },
-    { "PNM", 0, MagickString("Pf") },
+    { "PBM", 0, MagickString("P1") },
+    { "PGM", 0, MagickString("P2") },
+    { "PPM", 0, MagickString("P3") },
+    { "PBM", 0, MagickString("P4") },
+    { "PGM", 0, MagickString("P5") },
+    { "PPM", 0, MagickString("P6") },
+    { "PAM", 0, MagickString("P7") },
+    { "PFM", 0, MagickString("PF") },
+    { "PFM", 0, MagickString("Pf") },
     { "PS", 0, MagickString("%!") },
     { "PS", 0, MagickString("\004%!") },
     { "PS", 0, MagickString("\305\320\323\306") },
+    { "PSB", 0, MagickString("8BPB") },
     { "PSD", 0, MagickString("8BPS") },
     { "PWP", 0, MagickString("SFW95") },
     { "RAF", 0, MagickString("FUJIFILMCCD-RAW ") },
-    { "RAD", 0, MagickString("#?RADIANCE") },
-    { "RAD", 0, MagickString("VIEW= ") },
     { "RLE", 0, MagickString("\122\314") },
     { "SCT", 0, MagickString("CT") },
     { "SFW", 0, MagickString("SFW94") },
@@ -182,6 +185,7 @@ static const MagicMapInfo
     { "VICAR", 0, MagickString("LBLSIZE") },
     { "VICAR", 0, MagickString("NJPL1I") },
     { "VIFF", 0, MagickString("\253\001") },
+    { "WEBP", 8, MagickString("WEBP") },
     { "WMF", 0, MagickString("\327\315\306\232") },
     { "WMF", 0, MagickString("\001\000\011\000") },
     { "WPG", 0, MagickString("\377WPC") },
@@ -259,7 +263,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
   /*
     Search for magic tag.
   */
-  (void) LockSemaphoreInfo(magic_semaphore);
+  LockSemaphoreInfo(magic_semaphore);
   ResetLinkedListIterator(magic_list);
   p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   while (p != (const MagicInfo *) NULL)
@@ -273,7 +277,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
   if (p != (const MagicInfo *) NULL)
     (void) InsertValueInLinkedList(magic_list,0,
       RemoveElementByValueFromLinkedList(magic_list,p));
-  (void) UnlockSemaphoreInfo(magic_semaphore);
+  UnlockSemaphoreInfo(magic_semaphore);
   return(p);
 }
 
@@ -294,7 +298,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
 %  The magic of the GetMagicInfoList function is:
 %
 %      const MagicInfo **GetMagicInfoList(const char *pattern,
-%        unsigned long *number_aliases,ExceptionInfo *exception)
+%        size_t *number_aliases,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -328,7 +332,7 @@ static int MagicInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
-  unsigned long *number_aliases,ExceptionInfo *exception)
+  size_t *number_aliases,ExceptionInfo *exception)
 {
   const MagicInfo
     **aliases;
@@ -336,7 +340,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
   register const MagicInfo
     *p;
 
-  register long
+  register ssize_t
     i;
 
   /*
@@ -344,7 +348,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
   */
   assert(pattern != (char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
-  assert(number_aliases != (unsigned long *) NULL);
+  assert(number_aliases != (size_t *) NULL);
   *number_aliases=0;
   p=GetMagicInfo((const unsigned char *) NULL,0,exception);
   if (p == (const MagicInfo *) NULL)
@@ -356,7 +360,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
   /*
     Generate magic list.
   */
-  (void) LockSemaphoreInfo(magic_semaphore);
+  LockSemaphoreInfo(magic_semaphore);
   ResetLinkedListIterator(magic_list);
   p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   for (i=0; p != (const MagicInfo *) NULL; )
@@ -366,10 +370,10 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
       aliases[i++]=p;
     p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   }
-  (void) UnlockSemaphoreInfo(magic_semaphore);
+  UnlockSemaphoreInfo(magic_semaphore);
   qsort((void *) aliases,(size_t) i,sizeof(*aliases),MagicInfoCompare);
   aliases[i]=(MagicInfo *) NULL;
-  *number_aliases=(unsigned long) i;
+  *number_aliases=(size_t) i;
   return(aliases);
 }
 
@@ -389,7 +393,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
 %
 %  The format of the GetMagicList function is:
 %
-%      char **GetMagicList(const char *pattern,unsigned long *number_aliases,
+%      char **GetMagicList(const char *pattern,size_t *number_aliases,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -423,7 +427,7 @@ static int MagicCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetMagicList(const char *pattern,
-  unsigned long *number_aliases,ExceptionInfo *exception)
+  size_t *number_aliases,ExceptionInfo *exception)
 {
   char
     **aliases;
@@ -431,7 +435,7 @@ MagickExport char **GetMagicList(const char *pattern,
   register const MagicInfo
     *p;
 
-  register long
+  register ssize_t
     i;
 
   /*
@@ -439,7 +443,7 @@ MagickExport char **GetMagicList(const char *pattern,
   */
   assert(pattern != (char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
-  assert(number_aliases != (unsigned long *) NULL);
+  assert(number_aliases != (size_t *) NULL);
   *number_aliases=0;
   p=GetMagicInfo((const unsigned char *) NULL,0,exception);
   if (p == (const MagicInfo *) NULL)
@@ -448,7 +452,7 @@ MagickExport char **GetMagicList(const char *pattern,
     GetNumberOfElementsInLinkedList(magic_list)+1UL,sizeof(*aliases));
   if (aliases == (char **) NULL)
     return((char **) NULL);
-  (void) LockSemaphoreInfo(magic_semaphore);
+  LockSemaphoreInfo(magic_semaphore);
   ResetLinkedListIterator(magic_list);
   p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   for (i=0; p != (const MagicInfo *) NULL; )
@@ -458,10 +462,10 @@ MagickExport char **GetMagicList(const char *pattern,
       aliases[i++]=ConstantString(p->name);
     p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   }
-  (void) UnlockSemaphoreInfo(magic_semaphore);
+  UnlockSemaphoreInfo(magic_semaphore);
   qsort((void *) aliases,(size_t) i,sizeof(*aliases),MagicCompare);
   aliases[i]=(char *) NULL;
-  *number_aliases=(unsigned long) i;
+  *number_aliases=(size_t) i;
   return(aliases);
 }
 
@@ -524,14 +528,14 @@ static MagickBooleanType InitializeMagicList(ExceptionInfo *exception)
     {
       if (magic_semaphore == (SemaphoreInfo *) NULL)
         AcquireSemaphoreInfo(&magic_semaphore);
-      (void) LockSemaphoreInfo(magic_semaphore);
+      LockSemaphoreInfo(magic_semaphore);
       if ((magic_list == (LinkedListInfo *) NULL) &&
           (instantiate_magic == MagickFalse))
         {
           (void) LoadMagicLists(MagicFilename,exception);
           instantiate_magic=MagickTrue;
         }
-      (void) UnlockSemaphoreInfo(magic_semaphore);
+      UnlockSemaphoreInfo(magic_semaphore);
     }
   return(magic_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
 }
@@ -569,15 +573,15 @@ MagickExport MagickBooleanType ListMagicInfo(FILE *file,
   const MagicInfo
     **magic_info;
 
-  long
-    j;
-
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     number_aliases;
 
+  ssize_t
+    j;
+
   if (file == (const FILE *) NULL)
     file=stdout;
   magic_info=GetMagicInfoList("*",&number_aliases,exception);
@@ -585,7 +589,7 @@ MagickExport MagickBooleanType ListMagicInfo(FILE *file,
     return(MagickFalse);
   j=0;
   path=(const char *) NULL;
-  for (i=0; i < (long) number_aliases; i++)
+  for (i=0; i < (ssize_t) number_aliases; i++)
   {
     if (magic_info[i]->stealth != MagickFalse)
       continue;
@@ -593,29 +597,30 @@ MagickExport MagickBooleanType ListMagicInfo(FILE *file,
         (LocaleCompare(path,magic_info[i]->path) != 0))
       {
         if (magic_info[i]->path != (char *) NULL)
-          (void) fprintf(file,"\nPath: %s\n\n",magic_info[i]->path);
-        (void) fprintf(file,"Name      Offset Target\n");
-        (void) fprintf(file,"-------------------------------------------------"
+          (void) FormatLocaleFile(file,"\nPath: %s\n\n",magic_info[i]->path);
+        (void) FormatLocaleFile(file,"Name      Offset Target\n");
+        (void) FormatLocaleFile(file,
+          "-------------------------------------------------"
           "------------------------------\n");
       }
     path=magic_info[i]->path;
-    (void) fprintf(file,"%s",magic_info[i]->name);
-    for (j=(long) strlen(magic_info[i]->name); j <= 9; j++)
-      (void) fprintf(file," ");
-    (void) fprintf(file,"%6ld ",(long) magic_info[i]->offset);
+    (void) FormatLocaleFile(file,"%s",magic_info[i]->name);
+    for (j=(ssize_t) strlen(magic_info[i]->name); j <= 9; j++)
+      (void) FormatLocaleFile(file," ");
+    (void) FormatLocaleFile(file,"%6ld ",(long) magic_info[i]->offset);
     if (magic_info[i]->target != (char *) NULL)
       {
-        register long
+        register ssize_t
           j;
 
         for (j=0; magic_info[i]->target[j] != '\0'; j++)
           if (isprint((int) ((unsigned char) magic_info[i]->target[j])) != 0)
-            (void) fprintf(file,"%c",magic_info[i]->target[j]);
+            (void) FormatLocaleFile(file,"%c",magic_info[i]->target[j]);
           else
-            (void) fprintf(file,"\\%03o",(unsigned int)
+            (void) FormatLocaleFile(file,"\\%03o",(unsigned int)
               ((unsigned char) magic_info[i]->target[j]));
       }
-    (void) fprintf(file,"\n");
+    (void) FormatLocaleFile(file,"\n");
   }
   (void) fflush(file);
   magic_info=(const MagicInfo **) RelinquishMagickMemory((void *) magic_info);
@@ -639,7 +644,7 @@ MagickExport MagickBooleanType ListMagicInfo(FILE *file,
 %  The format of the LoadMagicList method is:
 %
 %      MagickBooleanType LoadMagicList(const char *xml,const char *filename,
-%        const unsigned long depth,ExceptionInfo *exception)
+%        const size_t depth,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -653,7 +658,7 @@ MagickExport MagickBooleanType ListMagicInfo(FILE *file,
 %
 */
 static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
-  const unsigned long depth,ExceptionInfo *exception)
+  const size_t depth,ExceptionInfo *exception)
 {
   char
     keyword[MaxTextExtent],
@@ -804,7 +809,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"offset") == 0)
           {
-            magic_info->offset=(MagickOffsetType) atol(token);
+            magic_info->offset=(MagickOffsetType) StringToLong(token);
             break;
           }
         break;
@@ -928,7 +933,7 @@ static MagickBooleanType LoadMagicLists(const char *filename,
   MagickStatusType
     status;
 
-  register long
+  register ssize_t
     i;
 
   /*
@@ -945,7 +950,7 @@ static MagickBooleanType LoadMagicLists(const char *filename,
           return(MagickFalse);
         }
     }
-  for (i=0; i < (long) (sizeof(MagicMap)/sizeof(*MagicMap)); i++)
+  for (i=0; i < (ssize_t) (sizeof(MagicMap)/sizeof(*MagicMap)); i++)
   {
     MagicInfo
       *magic_info;
@@ -1060,10 +1065,10 @@ MagickExport void MagicComponentTerminus(void)
 {
   if (magic_semaphore == (SemaphoreInfo *) NULL)
     AcquireSemaphoreInfo(&magic_semaphore);
-  (void) LockSemaphoreInfo(magic_semaphore);
+  LockSemaphoreInfo(magic_semaphore);
   if (magic_list != (LinkedListInfo *) NULL)
     magic_list=DestroyLinkedList(magic_list,DestroyMagicElement);
   instantiate_magic=MagickFalse;
-  (void) UnlockSemaphoreInfo(magic_semaphore);
+  UnlockSemaphoreInfo(magic_semaphore);
   DestroySemaphoreInfo(&magic_semaphore);
 }
