@@ -16,7 +16,7 @@
 %                              December 2001                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -58,7 +58,7 @@
 #include "magick/string_.h"
 #include "magick/thread_.h"
 #include "magick/thread-private.h"
-#include "magick/utility.h"
+#include "magick/utility-private.h"
 /*
   Define declarations.
 */
@@ -97,17 +97,17 @@ struct _RandomInfo
   SemaphoreInfo
     *semaphore;
 
-  long
+  ssize_t
     timestamp;
 
-  unsigned long
+  size_t
     signature;
 };
 
 /*
   External declarations.
 */
-#if defined(__APPLE__)
+#if defined(__APPLE__) && !defined(TARGET_OS_IPHONE)
 #include <crt_externs.h>
 #define environ (*_NSGetEnviron())
 #endif
@@ -172,7 +172,7 @@ MagickExport RandomInfo *AcquireRandomInfo(void)
     *key,
     *nonce;
 
-  random_info=(RandomInfo *) AcquireAlignedMemory(1,sizeof(*random_info));
+  random_info=(RandomInfo *) AcquireMagickMemory(sizeof(*random_info));
   if (random_info == (RandomInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   (void) ResetMagickMemory(random_info,0,sizeof(*random_info));
@@ -187,7 +187,7 @@ MagickExport RandomInfo *AcquireRandomInfo(void)
   random_info->semaphore=AllocateSemaphoreInfo();
   random_info->protocol_major=RandomProtocolMajorVersion;
   random_info->protocol_minor=RandomProtocolMinorVersion;
-  random_info->timestamp=(long) time(0);
+  random_info->timestamp=(ssize_t) time(0);
   random_info->signature=MagickSignature;
   /*
     Seed random nonce.
@@ -275,7 +275,7 @@ MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(random_info != (RandomInfo *) NULL);
   assert(random_info->signature == MagickSignature);
-  (void) LockSemaphoreInfo(random_info->semaphore);
+  LockSemaphoreInfo(random_info->semaphore);
   if (random_info->reservoir != (StringInfo *) NULL)
     random_info->reservoir=DestroyStringInfo(random_info->reservoir);
   if (random_info->nonce != (StringInfo *) NULL)
@@ -285,9 +285,9 @@ MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
       random_info->signature_info);
   (void) ResetMagickMemory(random_info->seed,0,sizeof(*random_info->seed));
   random_info->signature=(~MagickSignature);
-  (void) UnlockSemaphoreInfo(random_info->semaphore);
+  UnlockSemaphoreInfo(random_info->semaphore);
   DestroySemaphoreInfo(&random_info->semaphore);
-  random_info=(RandomInfo *) RelinquishAlignedMemory(random_info);
+  random_info=(RandomInfo *) RelinquishMagickMemory(random_info);
   return(random_info);
 }
 
@@ -315,7 +315,7 @@ MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
 %
 */
 
-#if !defined(__WINDOWS__)
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT)
 static ssize_t ReadRandom(int file,unsigned char *source,size_t length)
 {
   register unsigned char
@@ -347,9 +347,6 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
 {
 #define MaxEntropyExtent  64
 
-  long
-    pid;
-
   MagickThreadType
     tid;
 
@@ -357,21 +354,24 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     *chaos,
     *entropy;
 
-  unsigned long
+  size_t
     nanoseconds,
     seconds;
 
+  ssize_t
+    pid;
+
   /*
     Initialize random reservoir.
   */
   entropy=AcquireStringInfo(0);
-  (void) LockSemaphoreInfo(random_info->semaphore);
+  LockSemaphoreInfo(random_info->semaphore);
   chaos=AcquireStringInfo(sizeof(unsigned char *));
   SetStringInfoDatum(chaos,(unsigned char *) &entropy);
   ConcatenateStringInfo(entropy,chaos);
   SetStringInfoDatum(chaos,(unsigned char *) entropy);
   ConcatenateStringInfo(entropy,chaos);
-  pid=(long) getpid();
+  pid=(ssize_t) getpid();
   SetStringInfoLength(chaos,sizeof(pid));
   SetStringInfoDatum(chaos,(unsigned char *) &pid);
   ConcatenateStringInfo(entropy,chaos);
@@ -398,7 +398,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     struct timeval
       timer;
 
-    if (gettimeofday(&timer,0) == 0)
+    if (gettimeofday(&timer,(struct timezone *) NULL) == 0)
       {
         seconds=timer.tv_sec;
         nanoseconds=1000UL*timer.tv_usec;
@@ -454,14 +454,14 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
 #endif
     if (file != -1)
       (void) close(file);
-    (void) remove(filename);
+    (void) remove_utf8(filename);
     SetStringInfoLength(chaos,strlen(filename));
     SetStringInfoDatum(chaos,(unsigned char *) filename);
     ConcatenateStringInfo(entropy,chaos);
     filename=DestroyString(filename);
   }
 #endif
-#if defined(__WINDOWS__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
   {
     double
       seconds;
@@ -511,7 +511,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     */
     if (environ != (char **) NULL)
       {
-        register long
+        register ssize_t
           i;
 
         /*
@@ -527,7 +527,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     filename=AcquireString("/dev/urandom");
     device=StringToStringInfo(filename);
     device=DestroyStringInfo(device);
-    file=open(filename,O_RDONLY | O_BINARY);
+    file=open_utf8(filename,O_RDONLY | O_BINARY,0);
     filename=DestroyString(filename);
     if (file != -1)
       {
@@ -545,14 +545,14 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
         filename=AcquireString("/dev/random");
         device=StringToStringInfo(filename);
         device=DestroyStringInfo(device);
-        file=open(filename,O_RDONLY | O_BINARY);
+        file=open_utf8(filename,O_RDONLY | O_BINARY,0);
         filename=DestroyString(filename);
         if (file == -1)
           {
             filename=AcquireString("/dev/srandom");
             device=StringToStringInfo(filename);
             device=DestroyStringInfo(device);
-            file=open(filename,O_RDONLY | O_BINARY);
+            file=open_utf8(filename,O_RDONLY | O_BINARY,0);
           }
         if (file != -1)
           {
@@ -566,7 +566,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
   }
 #endif
   chaos=DestroyStringInfo(chaos);
-  (void) UnlockSemaphoreInfo(random_info->semaphore);
+  UnlockSemaphoreInfo(random_info->semaphore);
   return(entropy);
 }
 
@@ -791,14 +791,14 @@ MagickExport void SeedPseudoRandomGenerator(const unsigned long seed)
 
 static inline void IncrementRandomNonce(StringInfo *nonce)
 {
-  register long
+  register ssize_t
     i;
 
   unsigned char
     *datum;
 
   datum=GetStringInfoDatum(nonce);
-  for (i=(long) (GetStringInfoLength(nonce)-1); i != 0; i--)
+  for (i=(ssize_t) (GetStringInfoLength(nonce)-1); i != 0; i--)
   {
     datum[i]++;
     if (datum[i] != 0)
@@ -825,7 +825,7 @@ MagickExport void SetRandomKey(RandomInfo *random_info,const size_t length,
   assert(random_info != (RandomInfo *) NULL);
   if (length == 0)
     return;
-  (void) LockSemaphoreInfo(random_info->semaphore);
+  LockSemaphoreInfo(random_info->semaphore);
   signature_info=random_info->signature_info;
   datum=GetStringInfoDatum(random_info->reservoir);
   i=length;
@@ -859,7 +859,7 @@ MagickExport void SetRandomKey(RandomInfo *random_info,const size_t length,
       while (i-- != 0)
         p[i]=datum[i];
     }
-  (void) UnlockSemaphoreInfo(random_info->semaphore);
+  UnlockSemaphoreInfo(random_info->semaphore);
 }
 
 /*
