@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,16 +13,17 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick string methods.
+  MagickCore string methods.
 */
-#ifndef _MAGICK_STRING_H_
-#define _MAGICK_STRING_H_
+#ifndef _MAGICKCORE_STRING_H_
+#define _MAGICKCORE_STRING_H_
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
 #include <stdarg.h>
+#include <time.h>
 #include "magick/exception.h"
 
 typedef struct _StringInfo
@@ -34,32 +35,33 @@ typedef struct _StringInfo
     *datum;
 
   size_t
-    length;
-
-  unsigned long
+    length,
     signature;
 } StringInfo;
 
 extern MagickExport char
   *AcquireString(const char *),
   *CloneString(char **,const char *),
-  *ConstantString(char *),
+  *ConstantString(const char *),
   *DestroyString(char *),
   **DestroyStringList(char **),
   *EscapeString(const char *,const char),
   *FileToString(const char *,const size_t,ExceptionInfo *),
+  *GetEnvironmentValue(const char *),
+  *StringInfoToHexString(const StringInfo *),
+  *StringInfoToString(const StringInfo *),
   **StringToArgv(const char *,int *),
-  **StringToList(const char *),
-  *StringInfoToString(const StringInfo *);
+  *StringToken(const char *,char **),
+  **StringToList(const char *);
+
+extern MagickExport const char
+  *GetStringInfoPath(const StringInfo *);
 
 extern MagickExport double
-  StringToDouble(const char *,const double);
+  InterpretSiPrefixValue(const char *restrict,char **restrict);
 
-extern MagickExport long
-  FormatMagickString(char *,const size_t,const char *,...)
-    magick_attribute((format (printf,3,4))),
-  FormatMagickStringList(char *,const size_t,const char *,va_list)
-    magick_attribute((format (printf,3,0))),
+extern MagickExport int
+  CompareStringInfo(const StringInfo *,const StringInfo *),
   LocaleCompare(const char *,const char *),
   LocaleNCompare(const char *,const char *,const size_t);
 
@@ -67,15 +69,20 @@ extern MagickExport MagickBooleanType
   ConcatenateString(char **,const char *),
   SubstituteString(char **,const char *,const char *);
 
-extern MagickExport int
-  CompareStringInfo(const StringInfo *,const StringInfo *);
-
 extern MagickExport size_t
-  ConcatenateMagickString(char *,const char *,const size_t),
-  CopyMagickString(char *,const char *,const size_t);
+  ConcatenateMagickString(char *,const char *,const size_t)
+    magick_attribute((nonnull)),
+  CopyMagickString(char *,const char *,const size_t)
+    magick_attribute((nonnull)),
+  GetStringInfoLength(const StringInfo *);
+
+extern MagickExport ssize_t
+  FormatMagickSize(const MagickSizeType,const MagickBooleanType,char *),
+  FormatMagickTime(const time_t,const size_t,char *);
 
 extern MagickExport StringInfo
   *AcquireStringInfo(const size_t),
+  *BlobToStringInfo(const void *,const size_t),
   *CloneStringInfo(const StringInfo *),
   *ConfigureFileToStringInfo(const char *),
   *DestroyStringInfo(StringInfo *),
@@ -83,15 +90,20 @@ extern MagickExport StringInfo
   *SplitStringInfo(StringInfo *,const size_t),
   *StringToStringInfo(const char *);
 
+extern MagickExport unsigned char
+  *GetStringInfoDatum(const StringInfo *);
+
 extern MagickExport void
-  ConcatenateStringInfo(StringInfo *,const StringInfo *),
+  ConcatenateStringInfo(StringInfo *,const StringInfo *)
+    magick_attribute((nonnull)),
   LocaleLower(char *),
   LocaleUpper(char *),
   PrintStringInfo(FILE *file,const char *,const StringInfo *),
   ResetStringInfo(StringInfo *),
-  SetStringInfoDatum(StringInfo *,const unsigned char *),
   SetStringInfo(StringInfo *,const StringInfo *),
+  SetStringInfoDatum(StringInfo *,const unsigned char *),
   SetStringInfoLength(StringInfo *,const size_t),
+  SetStringInfoPath(StringInfo *,const char *),
   StripString(char *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
