@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,7 +46,9 @@
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
 #include "magick/token.h"
+#include "magick/token-private.h"
 #include "magick/utility.h"
 
 /*
@@ -60,13 +62,13 @@ struct _TokenInfo
   MagickStatusType
     flag;
 
-  long
+  ssize_t
     offset;
 
   char
     quote;
 
-  unsigned long
+  size_t
     signature;
 };
 
@@ -144,10 +146,11 @@ MagickExport TokenInfo *DestroyTokenInfo(TokenInfo *token_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetMagickToken() gets a token from the token stream.  A token is defined as a
-%  sequence of characters delimited by whitespace (e.g. clip-path), a sequence
-%  delimited with quotes (.e.g "Quote me"), or a sequence enclosed in
-%  parenthesis (e.g. rgb(0,0,0)).
+%  GetMagickToken() gets a token from the token stream.  A token is defined as
+%  a sequence of characters delimited by whitespace (e.g. clip-path), a
+%  sequence delimited with quotes (.e.g "Quote me"), or a sequence enclosed in
+%  parenthesis (e.g. rgb(0,0,0)).  GetMagickToken() also recognizes these
+%  separator characters: ':', '=', ',', and ';'.
 %
 %  The format of the GetMagickToken method is:
 %
@@ -170,9 +173,11 @@ MagickExport void GetMagickToken(const char *start,const char **end,char *token)
   register const char
     *p;
 
-  register long
+  register ssize_t
     i;
 
+  assert(start != (const char *) NULL);
+  assert(token != (char *) NULL);
   i=0;
   for (p=start; *p != '\0'; )
   {
@@ -224,7 +229,8 @@ MagickExport void GetMagickToken(const char *start,const char **end,char *token)
         char
           *q;
 
-        value=strtod(p,&q);
+        value=StringToDouble(p,&q);
+        (void) value;
         if ((p != q) && (*p != ','))
           {
             for ( ; (p < q) && (*p != ','); p++)
@@ -233,7 +239,7 @@ MagickExport void GetMagickToken(const char *start,const char **end,char *token)
               token[i++]=(*p++);
             break;
           }
-        if ((isalpha((int) ((unsigned char) *p)) == 0) &&
+        if ((*p != '\0') && (isalpha((int) ((unsigned char) *p)) == 0) &&
             (*p != *DirectorySeparator) && (*p != '#') && (*p != '<'))
           {
             token[i++]=(*p++);
@@ -242,7 +248,7 @@ MagickExport void GetMagickToken(const char *start,const char **end,char *token)
         for ( ; *p != '\0'; p++)
         {
           if (((isspace((int) ((unsigned char) *p)) != 0) || (*p == '=') ||
-              (*p == ',') || (*p == ':')) && (*(p-1) != '\\'))
+              (*p == ',') || (*p == ':') || (*p == ';')) && (*(p-1) != '\\'))
             break;
           if ((i > 0) && (*p == '<'))
             break;
@@ -271,7 +277,7 @@ MagickExport void GetMagickToken(const char *start,const char **end,char *token)
       offset=4;
       if (token[offset] == '#')
         offset++;
-      i=(long) strlen(token);
+      i=(ssize_t) strlen(token);
       (void) CopyMagickString(token,token+offset,MaxTextExtent);
       token[i-offset-1]='\0';
     }
@@ -324,12 +330,12 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
   */
   if (pattern == (char *) NULL)
     return(MagickTrue);
-  if (*pattern == '\0')
+  if (GetUTFCode(pattern) == 0)
     return(MagickTrue);
   if (LocaleCompare(pattern,"*") == 0)
     return(MagickTrue);
   p=pattern+strlen(pattern)-1;
-  if ((*p == ']') && (strchr(pattern,'[') != (char *) NULL))
+  if ((GetUTFCode(p) == ']') && (strchr(pattern,'[') != (char *) NULL))
     {
       ExceptionInfo
         *exception;
@@ -343,7 +349,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
       image_info=AcquireImageInfo();
       (void) CopyMagickString(image_info->filename,pattern,MaxTextExtent);
       exception=AcquireExceptionInfo();
-      (void) SetImageInfo(image_info,MagickTrue,exception);
+      (void) SetImageInfo(image_info,0,exception);
       exception=DestroyExceptionInfo(exception);
       if (LocaleCompare(image_info->filename,pattern) != 0)
         {
@@ -356,101 +362,100 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
     Evaluate glob expression.
   */
   done=MagickFalse;
-  while ((*pattern != '\0') && (done == MagickFalse))
+  while ((GetUTFCode(pattern) != 0) && (done == MagickFalse))
   {
-    if (*expression == '\0')
-      if ((*pattern != '{') && (*pattern != '*'))
+    if (GetUTFCode(expression) == 0)
+      if ((GetUTFCode(pattern) != '{') && (GetUTFCode(pattern) != '*'))
         break;
-    switch (*pattern)
+    switch (GetUTFCode(pattern))
     {
-      case '\\':
-      {
-        pattern++;
-        if (*pattern != '\0')
-          pattern++;
-        break;
-      }
       case '*':
       {
         MagickBooleanType
           status;
 
-        pattern++;
         status=MagickFalse;
-        while ((*expression != '\0') && (status == MagickFalse))
-          status=GlobExpression(expression++,pattern,case_insensitive);
+        pattern+=GetUTFOctets(pattern);
+        while ((GetUTFCode(expression) != 0) && (status == MagickFalse))
+        {
+          status=GlobExpression(expression,pattern,case_insensitive);
+          expression+=GetUTFOctets(expression);
+        }
         if (status != MagickFalse)
           {
-            while (*expression != '\0')
-              expression++;
-            while (*pattern != '\0')
-              pattern++;
+            while (GetUTFCode(expression) != 0)
+              expression+=GetUTFOctets(expression);
+            while (GetUTFCode(pattern) != 0)
+              pattern+=GetUTFOctets(pattern);
           }
         break;
       }
       case '[':
       {
-        char
+        int
           c;
 
-        pattern++;
+        pattern+=GetUTFOctets(pattern);
         for ( ; ; )
         {
-          if ((*pattern == '\0') || (*pattern == ']'))
+          if ((GetUTFCode(pattern) == 0) || (GetUTFCode(pattern) == ']'))
             {
               done=MagickTrue;
               break;
             }
-          if (*pattern == '\\')
+          if (GetUTFCode(pattern) == '\\')
             {
-              pattern++;
-              if (*pattern == '\0')
+              pattern+=GetUTFOctets(pattern);
+              if (GetUTFCode(pattern) == 0)
                 {
                   done=MagickTrue;
                   break;
                 }
              }
-          if (*(pattern+1) == '-')
+          if (GetUTFCode(pattern+GetUTFOctets(pattern)) == '-')
             {
-              c=(*pattern);
-              pattern+=2;
-              if (*pattern == ']')
+              c=GetUTFCode(pattern);
+              pattern+=GetUTFOctets(pattern);
+              pattern+=GetUTFOctets(pattern);
+              if (GetUTFCode(pattern) == ']')
                 {
                   done=MagickTrue;
                   break;
                 }
-              if (*pattern == '\\')
+              if (GetUTFCode(pattern) == '\\')
                 {
-                  pattern++;
-                  if (*pattern == '\0')
+                  pattern+=GetUTFOctets(pattern);
+                  if (GetUTFCode(pattern) == 0)
                     {
                       done=MagickTrue;
                       break;
                     }
                 }
-              if ((*expression < c) || (*expression > *pattern))
+              if ((GetUTFCode(expression) < c) ||
+                  (GetUTFCode(expression) > GetUTFCode(pattern)))
                 {
-                  pattern++;
+                  pattern+=GetUTFOctets(pattern);
                   continue;
                 }
             }
           else
-            if (*pattern != *expression)
+            if (GetUTFCode(pattern) != GetUTFCode(expression))
               {
-                pattern++;
+                pattern+=GetUTFOctets(pattern);
                 continue;
               }
-          pattern++;
-          while ((*pattern != ']') && (*pattern != '\0'))
+          pattern+=GetUTFOctets(pattern);
+          while ((GetUTFCode(pattern) != ']') && (GetUTFCode(pattern) != 0))
           {
-            if ((*pattern == '\\') && (*(pattern+1) != '\0'))
-              pattern++;
-            pattern++;
+            if ((GetUTFCode(pattern) == '\\') &&
+                (GetUTFCode(pattern+GetUTFOctets(pattern)) > 0))
+              pattern+=GetUTFOctets(pattern);
+            pattern+=GetUTFOctets(pattern);
           }
-          if (*pattern != '\0')
+          if (GetUTFCode(pattern) != 0)
             {
-              pattern++;
-              expression++;
+              pattern+=GetUTFOctets(pattern);
+              expression+=GetUTFOctets(expression);
             }
           break;
         }
@@ -458,8 +463,8 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
       }
       case '?':
       {
-        pattern++;
-        expression++;
+        pattern+=GetUTFOctets(pattern);
+        expression+=GetUTFOctets(expression);
         break;
       }
       case '{':
@@ -467,22 +472,23 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
         register const char
           *p;
 
-        pattern++;
-        while ((*pattern != '}') && (*pattern != '\0'))
+        pattern+=GetUTFOctets(pattern);
+        while ((GetUTFCode(pattern) != '}') && (GetUTFCode(pattern) != 0))
         {
           p=expression;
           match=MagickTrue;
-          while ((*p != '\0') && (*pattern != '\0') &&
-                 (*pattern != ',') && (*pattern != '}') &&
+          while ((GetUTFCode(p) != 0) && (GetUTFCode(pattern) != 0) &&
+                 (GetUTFCode(pattern) != ',') && (GetUTFCode(pattern) != '}') &&
                  (match != MagickFalse))
           {
-            if (*pattern == '\\')
-              pattern++;
-            match=(*pattern == *p) ? MagickTrue : MagickFalse;
-            p++;
-            pattern++;
+            if (GetUTFCode(pattern) == '\\')
+              pattern+=GetUTFOctets(pattern);
+            match=(GetUTFCode(pattern) == GetUTFCode(p)) ? MagickTrue :
+              MagickFalse;
+            p+=GetUTFOctets(p);
+            pattern+=GetUTFOctets(pattern);
           }
-          if (*pattern == '\0')
+          if (GetUTFCode(pattern) == 0)
             {
               match=MagickFalse;
               done=MagickTrue;
@@ -492,61 +498,71 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
             if (match != MagickFalse)
               {
                 expression=p;
-                while ((*pattern != '}') && (*pattern != '\0'))
+                while ((GetUTFCode(pattern) != '}') &&
+                       (GetUTFCode(pattern) != 0))
                 {
-                  pattern++;
-                  if (*pattern == '\\')
+                  pattern+=GetUTFOctets(pattern);
+                  if (GetUTFCode(pattern) == '\\')
                     {
-                      pattern++;
-                      if (*pattern == '}')
-                        pattern++;
+                      pattern+=GetUTFOctets(pattern);
+                      if (GetUTFCode(pattern) == '}')
+                        pattern+=GetUTFOctets(pattern);
                     }
                 }
               }
             else
               {
-                while ((*pattern != '}') && (*pattern != ',') &&
-                       (*pattern != '\0'))
+                while ((GetUTFCode(pattern) != '}') &&
+                       (GetUTFCode(pattern) != ',') &&
+                       (GetUTFCode(pattern) != 0))
                 {
-                  pattern++;
-                  if (*pattern == '\\')
+                  pattern+=GetUTFOctets(pattern);
+                  if (GetUTFCode(pattern) == '\\')
                     {
-                      pattern++;
-                      if ((*pattern == '}') || (*pattern == ','))
-                        pattern++;
+                      pattern+=GetUTFOctets(pattern);
+                      if ((GetUTFCode(pattern) == '}') ||
+                          (GetUTFCode(pattern) == ','))
+                        pattern+=GetUTFOctets(pattern);
                     }
                 }
               }
-            if (*pattern != '\0')
-              pattern++;
+            if (GetUTFCode(pattern) != 0)
+              pattern+=GetUTFOctets(pattern);
           }
         break;
       }
+      case '\\':
+      {
+        pattern+=GetUTFOctets(pattern);
+        if (GetUTFCode(pattern) == 0)
+          break;
+      }
       default:
       {
         if (case_insensitive != MagickFalse)
           {
-            if (tolower((int) ((unsigned char) *expression)) !=
-                tolower((int) ((unsigned char) *pattern)))
+            if (tolower((int) GetUTFCode(expression)) !=
+                tolower((int) GetUTFCode(pattern)))
               {
                 done=MagickTrue;
                 break;
               }
           }
         else
-          if (*expression != *pattern)
+          if (GetUTFCode(expression) != GetUTFCode(pattern))
             {
               done=MagickTrue;
               break;
             }
-        expression++;
-        pattern++;
+        expression+=GetUTFOctets(expression);
+        pattern+=GetUTFOctets(pattern);
       }
     }
   }
-  while (*pattern == '*') 
-    pattern++;
-  match=(*expression == '\0') && (*pattern == '\0') ? MagickTrue : MagickFalse;
+  while (GetUTFCode(pattern) == '*')
+    pattern+=GetUTFOctets(pattern);
+  match=(GetUTFCode(expression) == 0) && (GetUTFCode(pattern) == 0) ?
+    MagickTrue : MagickFalse;
   return(match);
 }
 
@@ -756,21 +772,21 @@ MagickExport MagickBooleanType IsGlob(const char *path)
 #define IN_QUOTE 2
 #define IN_OZONE 3
 
-static long sindex(int c,const char *string)
+static ssize_t sindex(int c,const char *string)
 {
   register const char
     *p;
 
   for (p=string; *p != '\0'; p++)
     if (c == (int) (*p))
-      return(p-string);
+      return((ssize_t) (p-string));
   return(-1);
 }
 
 static void StoreToken(TokenInfo *token_info,char *string,
   size_t max_token_length,int c)
 {
-  register long
+  register ssize_t
     i;
 
   if ((token_info->offset < 0) ||
@@ -805,7 +821,7 @@ MagickExport int Tokenizer(TokenInfo *token_info,const unsigned flag,
   int
     c;
 
-  register long
+  register ssize_t
     i;
 
   *breaker='\0';
